
-------------
CREATE OR REPLACE TYPE ssstest_user.ssstest_expondist_ty AS TABLE OF INTEGER
/

GRANT EXECUTE ON ssstest_user.ssstest_expondist_ty TO scott;

CREATE OR REPLACE FUNCTION ssstest_user.fnc_ssstest_expondist
	( p_lambda IN INTEGER, p_return_array_size INTEGER, p_pdf_cdf_flg VARCHAR2 DEFAULT 'CDF') 
RETURN ssstest_user.ssstest_expondist_ty
-- 1. Generating discrete RV distribution:
-- 	1.1 - Build an array of probabilities p0, p1, ... pN for x(i)=0,1,2,3,4,5
--	      that user logs in and execute a query in precisely i second from the start of the interval
-- 	1.2 - Generate an array (of requested size) of pseudo-random variable U with distribution U(0,1)
--	      Size of an array should be equal to the number of "concurrent" users in the current run
-- 	1.3 - Set and return X as an array of a specified size
-- 2. Generating continuous RV distribution (for CDF only?):
AS
	p_pdf_cdf_flg_trim	VARCHAR2(10);
	TYPE probability_ty IS VARRAY(3100) OF NUMBER;
	p			probability_ty := probability_ty();
	p_sum			NUMBER;
	t_uniformdist		NUMBER;
	t_expondists		ssstest_user.ssstest_expondist_ty := ssstest_user.ssstest_expondist_ty();
	t_exp			INT;
	e_wrongFlag		EXCEPTION;
	glb_wrongFlag		VARCHAR2(100) := 'Wrong Flag: allowed values are pdf, pdf2, pdf3, cdf or cdf2. Passed=';
	PRAGMA EXCEPTION_INIT( e_wrongFlag, -20100);
	t_min_product		NUMBER;
	t_limit			NUMBER;
	i			INT;
	j			INT;
	t_break			BOOLEAN := FALSE;
	t_sum_arrivals		INT := 0;
BEGIN
	p_pdf_cdf_flg_trim := Upper(Trim(p_pdf_cdf_flg));
	IF( p_pdf_cdf_flg_trim = 'PDF' OR p_pdf_cdf_flg_trim = 'CDF') THEN
		-- 1.1
		-- FOR i IN 1..31 LOOP
		FOR i IN 1..31 LOOP
			p.EXTEND;
			IF( p_pdf_cdf_flg_trim = 'PDF') THEN
				p(i) := p_lambda * exp( -1*p_lambda * i);
			-- Generating Poisson RV (Ross, page 50):
			ELSIF( p_pdf_cdf_flg_trim = 'PDF_temp') THEN
				IF( i=1) THEN p(1) := exp(-1*p_lambda); 
				ELSE  p(i) := p_lambda * p(i-1)/(i+1);
				END IF;
			ELSIF( p_pdf_cdf_flg_trim = 'CDF') THEN
				p(i) := 1 - exp( -1*p_lambda * i);
			ELSE
				RAISE e_wrongFlag;
			END IF;
		END LOOP;

		-- 1.2 and 1.3: Inverse Transform Technique: exponentially distributed interarrival times:
		FOR i IN 1..p_return_array_size LOOP
		   t_uniformdist := dbms_random.value;
		   IF ( p_pdf_cdf_flg_trim = 'PDF') THEN
			p_sum := 0; j := 1; t_break := TRUE; t_exp := 0; -- -1;
			WHILE (j <= 31 AND t_break) LOOP
				p_sum := p_sum + p(j);
				IF( t_uniformdist < p_sum) THEN 
					t_exp := j;	-- -1; 
					t_break := FALSE;
				END IF;
				j := j + 1;
			END LOOP;
			-- if not up to 29, make it 30:
			-- IF( t_exp = -1) THEN t_exp := 30; END IF;
		   ELSIF ( p_pdf_cdf_flg_trim = 'PDF_temp') THEN
			IF( t_uniformdist < p(1)) 
				THEN t_exp := 0; 
			ELSIF( t_uniformdist >= p(1) 
			   AND t_uniformdist < p(1)+p(2)) 
				THEN t_exp := 1;
			ELSIF( t_uniformdist >= p(1)+p(2) 
			   AND t_uniformdist < p(1)+p(2)+p(3)) 
				THEN t_exp := 2;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3) 
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)) 
				THEN t_exp := 3;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)) 
				THEN t_exp := 4;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)) 
				THEN t_exp := 5;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)) 
				THEN t_exp := 6;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)) 
				THEN t_exp := 7;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)) 
				THEN t_exp := 8;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)) 
				THEN t_exp := 9;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)) 
				THEN t_exp := 10;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)) 
				THEN t_exp := 11;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)) 
				THEN t_exp := 12;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)) 
				THEN t_exp := 13;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)) 
				THEN t_exp := 14;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)) 
				THEN t_exp := 15;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)) 
				THEN t_exp := 16;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)) 
				THEN t_exp := 17;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)) 
				THEN t_exp := 18;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)) 
				THEN t_exp := 19;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)) 
				THEN t_exp := 20;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)) 
				THEN t_exp := 21;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)) 
				THEN t_exp := 22;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)) 
				THEN t_exp := 23;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)) 
				THEN t_exp := 24;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)+p(26)) 
				THEN t_exp := 25;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)+p(26)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)+p(26)+p(27)) 
				THEN t_exp := 26;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)+p(26)+p(27)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)+p(26)+p(27)+p(28)) 
				THEN t_exp := 27;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)+p(26)+p(27)+p(28)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)+p(26)+p(27)+p(28)+p(29)) 
				THEN t_exp := 28;
			ELSIF( t_uniformdist >= p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)+p(26)+p(27)+p(28)+p(29)
			   AND t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)+p(26)+p(27)+p(28)+p(29)+p(30)) 
				THEN t_exp := 29;
			ELSE t_exp := 30;
			END IF;
		  -- Same as above, using shortcut of ordered set 0,1...30:
		   ELSIF ( p_pdf_cdf_flg_trim = 'PDF_temp') THEN
			IF( t_uniformdist < p(1)) 
				THEN t_exp := 0; 
			ELSIF( t_uniformdist < p(1)+p(2)) 
				THEN t_exp := 1;
			ELSIF( t_uniformdist < p(1)+p(2)+p(3)) 
				THEN t_exp := 2;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)) 
				THEN t_exp := 3;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)) 
				THEN t_exp := 4;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)) 
				THEN t_exp := 5;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)) 
				THEN t_exp := 6;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)) 
				THEN t_exp := 7;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)) 
				THEN t_exp := 8;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)) 
				THEN t_exp := 9;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)) 
				THEN t_exp := 10;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)) 
				THEN t_exp := 11;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)) 
				THEN t_exp := 12;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)) 
				THEN t_exp := 13;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)) 
				THEN t_exp := 14;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)) 
				THEN t_exp := 15;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)) 
				THEN t_exp := 16;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)) 
				THEN t_exp := 17;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)) 
				THEN t_exp := 18;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)) 
				THEN t_exp := 19;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)) 
				THEN t_exp := 20;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)) 
				THEN t_exp := 21;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)) 
				THEN t_exp := 22;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)) 
				THEN t_exp := 23;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)) 
				THEN t_exp := 24;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)+p(26)) 
				THEN t_exp := 25;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)+p(26)+p(27)) 
				THEN t_exp := 26;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)+p(26)+p(27)+p(28)) 
				THEN t_exp := 27;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)+p(26)+p(27)+p(28)+p(29)) 
				THEN t_exp := 28;
			ELSIF( t_uniformdist <  p(1)+p(2)+p(3)+p(4)+p(5)+p(6)+p(7)+p(8)+p(9)+p(10)+
						p(11)+p(12)+p(13)+p(14)+p(15)+p(16)+p(17)+p(18)+p(19)+p(20)+
						p(21)+p(22)+p(23)+p(24)+p(25)+p(26)+p(27)+p(28)+p(29)+p(30)) 
				THEN t_exp := 29;
			ELSE t_exp := 30;
			END IF;
		   ELSIF ( p_pdf_cdf_flg_trim = 'CDF') THEN
			IF ( t_uniformdist <= p(1)) THEN t_exp := 0; 
			ELSIF( t_uniformdist > p(1)  AND t_uniformdist <= p(2))  THEN t_exp := 1;
			ELSIF( t_uniformdist > p(2)  AND t_uniformdist <= p(3))  THEN t_exp := 2;
			ELSIF( t_uniformdist > p(3)  AND t_uniformdist <= p(4))  THEN t_exp := 3;
			ELSIF( t_uniformdist > p(4)  AND t_uniformdist <= p(5))  THEN t_exp := 4;
			ELSIF( t_uniformdist > p(5)  AND t_uniformdist <= p(6))  THEN t_exp := 5;
			ELSIF( t_uniformdist > p(6)  AND t_uniformdist <= p(7))  THEN t_exp := 6;
			ELSIF( t_uniformdist > p(7)  AND t_uniformdist <= p(8))  THEN t_exp := 7;
			ELSIF( t_uniformdist > p(8)  AND t_uniformdist <= p(9))  THEN t_exp := 8;
			ELSIF( t_uniformdist > p(9)  AND t_uniformdist <= p(10)) THEN t_exp := 9;
			ELSIF( t_uniformdist > p(10) AND t_uniformdist <= p(11)) THEN t_exp :=10;
			ELSIF( t_uniformdist > p(11) AND t_uniformdist <= p(12)) THEN t_exp :=11;
			ELSIF( t_uniformdist > p(12) AND t_uniformdist <= p(13)) THEN t_exp :=12;
			ELSIF( t_uniformdist > p(13) AND t_uniformdist <= p(14)) THEN t_exp :=13;
			ELSIF( t_uniformdist > p(14) AND t_uniformdist <= p(15)) THEN t_exp :=14;
			ELSIF( t_uniformdist > p(15) AND t_uniformdist <= p(16)) THEN t_exp :=15;
			ELSIF( t_uniformdist > p(16) AND t_uniformdist <= p(17)) THEN t_exp :=16;
			ELSIF( t_uniformdist > p(17) AND t_uniformdist <= p(18)) THEN t_exp :=17;
			ELSIF( t_uniformdist > p(18) AND t_uniformdist <= p(19)) THEN t_exp :=18;
			ELSIF( t_uniformdist > p(19) AND t_uniformdist <= p(20)) THEN t_exp :=19;
			ELSIF( t_uniformdist > p(20) AND t_uniformdist <= p(21)) THEN t_exp :=20;
			ELSIF( t_uniformdist > p(21) AND t_uniformdist <= p(22)) THEN t_exp :=21;
			ELSIF( t_uniformdist > p(22) AND t_uniformdist <= p(23)) THEN t_exp :=22;
			ELSIF( t_uniformdist > p(23) AND t_uniformdist <= p(24)) THEN t_exp :=23;
			ELSIF( t_uniformdist > p(24) AND t_uniformdist <= p(25)) THEN t_exp :=24;
			ELSIF( t_uniformdist > p(25) AND t_uniformdist <= p(26)) THEN t_exp :=25;
			ELSIF( t_uniformdist > p(26) AND t_uniformdist <= p(27)) THEN t_exp :=26;
			ELSIF( t_uniformdist > p(27) AND t_uniformdist <= p(28)) THEN t_exp :=27;
			ELSIF( t_uniformdist > p(28) AND t_uniformdist <= p(29)) THEN t_exp :=28;
			ELSIF( t_uniformdist > p(29) AND t_uniformdist <= p(30)) THEN t_exp :=29;
			ELSE t_exp := 30;
			END IF;
		   END IF;

			t_expondists.EXTEND;
			t_expondists(i) := t_exp;
		END LOOP;
	ELSIF( p_pdf_cdf_flg_trim = 'PDF_temp') THEN
		-- 1.2 and 1.3: Inverse Transform Technique: exponentially distributed interarrival times:
		FOR i IN 1..p_return_array_size LOOP
			p.EXTEND;
		   	t_uniformdist := dbms_random.value;
			-- f(x)=lambda*exp(-lambda*x) and x=f-1(u), so x=(ln(lambda)-ln(u))/lambda:
			p(i) := (ln(p_lambda) - ln(t_uniformdist))/p_lambda;
			IF( p(i) < 0) THEN p(i) := 0; END IF;
			t_expondists.EXTEND;
			t_expondists(i) := Round(p(i),0);
		END LOOP;
	ELSIF( p_pdf_cdf_flg_trim = 'CDF2') THEN
		FOR i IN 1..p_return_array_size LOOP
			t_expondists.EXTEND;
			t_expondists(i) := Round( -1/p_lambda * ln( 1 - dbms_random.value), 0);
		END LOOP;
	ELSIF( p_pdf_cdf_flg_trim = 'PDF2') THEN
		-- Generating continuous Poisson distributed RV - number of arrivals (Ross, p. 66 ):
		t_limit := exp(-1*p_lambda);
		i := 1;
		WHILE( t_sum_arrivals < p_return_array_size) LOOP
			t_expondists.EXTEND;
			t_min_product := 1; j := 1;
			WHILE ( t_min_product >= t_limit AND t_break=FALSE) LOOP
				t_min_product := t_min_product * dbms_random.value;
				j := j + 1;
				IF ( j > 1000) THEN t_break:=TRUE; END IF;
			END LOOP;
			-- raise_application_error( -20101, 'i='||i||',t_min_product='||t_min_product||', t_limit='||t_limit);
			t_sum_arrivals := t_sum_arrivals + (j-2);
			t_expondists(i) := j-2;		-- -1;
			IF( t_break) THEN t_expondists(i) := 1000; END IF;
			i := i + 1;
		END LOOP;
	ELSIF( p_pdf_cdf_flg_trim = 'PDF3') THEN
		-- Generating descrete Poisson distributed RV - number of arrivals (Ross, p. 50):
		FOR i IN 1..30 LOOP
			p.EXTEND;
			IF( i=1) THEN p(1) := exp(-1*p_lambda); 
			ELSE  p(i) := p_lambda * p(i-1)/(i+1);
			END IF;
		END LOOP;

		i := 1;
		WHILE( t_sum_arrivals < p_return_array_size) LOOP
		   	t_expondists.EXTEND;
			p_sum := 0; j := 1; t_break := TRUE; t_exp := -1;
			WHILE (j <= 30 AND t_break) LOOP
				p_sum := p_sum + p(j);
				IF( dbms_random.value < p_sum) THEN 
					t_exp := j -1; 
				 	t_break := FALSE;
				END IF;
				j := j + 1;
			END LOOP;
			IF( t_exp = -1) THEN t_exp := 0; END IF;
			t_sum_arrivals := t_sum_arrivals + t_exp;
			t_expondists(i) := t_exp;
			i := i + 1;
		END LOOP;
		-- There's a possibility that last or one before last number of arrivals happens to be >1, which might make total number 
		-- of arrivals for the run greater than requested, so chop-off:
		t_expondists(i-1) := t_expondists(i-1) - (t_sum_arrivals-p_return_array_size);
	ELSE
		RAISE e_wrongFlag;
	END IF;
	RETURN t_expondists;
EXCEPTION
	WHEN e_wrongFlag THEN
		RAISE_APPLICATION_ERROR( -20100, glb_wrongFlag || p_pdf_cdf_flg);
END fnc_ssstest_expondist;
/

sho err
GRANT EXECUTE ON ssstest_user.fnc_ssstest_expondist TO scott;

set serveroutput on size 1000000 feed off
PROMPT *** pdf - 1000:
SELECT
         Substr( Sum( Decode( i, 0, 1, 0)),1,3) "0"
        ,Substr( Sum( Decode( i, 1, 1, 0)),1,3) "1"
        ,Substr( Sum( Decode( i, 2, 1, 0)),1,3) "2"
        ,Substr( Sum( Decode( i, 3, 1, 0)),1,3) "3"
        ,Substr( Sum( Decode( i, 4, 1, 0)),1,3) "4"
        ,Substr( Sum( Decode( i, 5, 1, 0)),1,3) "5"
        ,Substr( Sum( Decode( i, 6, 1, 0)),1,3) "6"
        ,Substr( Sum( Decode( i, 7, 1, 0)),1,3) "7"
        ,Substr( Sum( Decode( i, 8, 1, 0)),1,3) "8"
        ,Substr( Sum( Decode( i, 9, 1, 0)),1,3) "9"
        ,Substr( Sum( Decode( i, 10, 1, 0)),1,3) "10"
        ,Substr( Sum( Decode( i, 11, 1, 0)),1,3) "11"
        ,Substr( Sum( Decode( i, 12, 1, 0)),1,3) "12"
        ,Substr( Sum( Decode( i, 13, 1, 0)),1,3) "13"
        ,Substr( Sum( Decode( i, 14, 1, 0)),1,3) "14"
        ,Substr( Sum( Decode( i, 15, 1, 0)),1,3) "15"
        ,Substr( Sum( Decode( i, 16, 1, 0)),1,3) "16"
        ,Substr( Sum( Decode( i, 17, 1, 0)),1,3) "17"
        ,Substr( Sum( Decode( i, 18, 1, 0)),1,3) "18"
        ,Substr( Sum( Decode( i, 19, 1, 0)),1,3) "19"
        ,Substr( Sum( Decode( i, 20, 1, 0)),1,3) "20"
        ,Substr( Sum( Decode( i, 21, 1, 0)),1,3) "21"
        ,Substr( Sum( Decode( i, 22, 1, 0)),1,3) "22"
        ,Substr( Sum( Decode( i, 23, 1, 0)),1,3) "23"
        ,Substr( Sum( Decode( i, 24, 1, 0)),1,3) "24"
        ,Substr( Sum( Decode( i, 25, 1, 0)),1,3) "25"
        ,Substr( Sum( Decode( i, 26, 1, 0)),1,3) "26"
        ,Substr( Sum( Decode( i, 27, 1, 0)),1,3) "27"
        ,Substr( Sum( Decode( i, 28, 1, 0)),1,3) "28"
        ,Substr( Sum( Decode( i, 29, 1, 0)),1,3) "29"
        ,Substr( Sum( Decode( i, 30, 1, 0)),1,3) "30"
        ,Substr( Count(i),1,5) "total"
FROM    (SELECT column_value i FROM Table( ssstest_user.fnc_ssstest_expondist( 1/3, 1000, 'pdf')));

PROMPT *** pdf2 - 1000:
SELECT
         Substr( Sum( Decode( i, 0, 1, 0)),1,3) "0"
        ,Substr( Sum( Decode( i, 1, 1, 0)),1,3) "1"
        ,Substr( Sum( Decode( i, 2, 1, 0)),1,3) "2"
        ,Substr( Sum( Decode( i, 3, 1, 0)),1,3) "3"
        ,Substr( Sum( Decode( i, 4, 1, 0)),1,3) "4"
        ,Substr( Sum( Decode( i, 5, 1, 0)),1,3) "5"
        ,Substr( Sum( Decode( i, 6, 1, 0)),1,3) "6"
        ,Substr( Sum( Decode( i, 7, 1, 0)),1,3) "7"
        ,Substr( Sum( Decode( i, 8, 1, 0)),1,3) "8"
        ,Substr( Sum( Decode( i, 9, 1, 0)),1,3) "9"
        ,Substr( Sum( Decode( i, 10, 1, 0)),1,3) "10"
        ,Substr( Sum( Decode( i, 11, 1, 0)),1,3) "11"
        ,Substr( Sum( Decode( i, 12, 1, 0)),1,3) "12"
        ,Substr( Sum( Decode( i, 13, 1, 0)),1,3) "13"
        ,Substr( Sum( Decode( i, 14, 1, 0)),1,3) "14"
        ,Substr( Sum( Decode( i, 15, 1, 0)),1,3) "15"
        ,Substr( Sum( Decode( i, 16, 1, 0)),1,3) "16"
        ,Substr( Sum( Decode( i, 17, 1, 0)),1,3) "17"
        ,Substr( Sum( Decode( i, 18, 1, 0)),1,3) "18"
        ,Substr( Sum( Decode( i, 19, 1, 0)),1,3) "19"
        ,Substr( Sum( Decode( i, 20, 1, 0)),1,3) "20"
        ,Substr( Sum( Decode( i, 21, 1, 0)),1,3) "21"
        ,Substr( Sum( Decode( i, 22, 1, 0)),1,3) "22"
        ,Substr( Sum( Decode( i, 23, 1, 0)),1,3) "23"
        ,Substr( Sum( Decode( i, 24, 1, 0)),1,3) "24"
        ,Substr( Sum( Decode( i, 25, 1, 0)),1,3) "25"
        ,Substr( Sum( Decode( i, 26, 1, 0)),1,3) "26"
        ,Substr( Sum( Decode( i, 27, 1, 0)),1,3) "27"
        ,Substr( Sum( Decode( i, 28, 1, 0)),1,3) "28"
        ,Substr( Sum( Decode( i, 29, 1, 0)),1,3) "29"
        ,Substr( Sum( Decode( i, 30, 1, 0)),1,3) "30"
        ,Substr( Count(i),1,5) "total"
FROM    (SELECT column_value i FROM Table( ssstest_user.fnc_ssstest_expondist( 1/3, 1000, 'pdf2')));

PROMPT *** pdf3 - 1000:
SELECT
         Substr( Sum( Decode( i, 0, 1, 0)),1,3) "0"
        ,Substr( Sum( Decode( i, 1, 1, 0)),1,3) "1"
        ,Substr( Sum( Decode( i, 2, 1, 0)),1,3) "2"
        ,Substr( Sum( Decode( i, 3, 1, 0)),1,3) "3"
        ,Substr( Sum( Decode( i, 4, 1, 0)),1,3) "4"
        ,Substr( Sum( Decode( i, 5, 1, 0)),1,3) "5"
        ,Substr( Sum( Decode( i, 6, 1, 0)),1,3) "6"
        ,Substr( Sum( Decode( i, 7, 1, 0)),1,3) "7"
        ,Substr( Sum( Decode( i, 8, 1, 0)),1,3) "8"
        ,Substr( Sum( Decode( i, 9, 1, 0)),1,3) "9"
        ,Substr( Sum( Decode( i, 10, 1, 0)),1,3) "10"
        ,Substr( Sum( Decode( i, 11, 1, 0)),1,3) "11"
        ,Substr( Sum( Decode( i, 12, 1, 0)),1,3) "12"
        ,Substr( Sum( Decode( i, 13, 1, 0)),1,3) "13"
        ,Substr( Sum( Decode( i, 14, 1, 0)),1,3) "14"
        ,Substr( Sum( Decode( i, 15, 1, 0)),1,3) "15"
        ,Substr( Sum( Decode( i, 16, 1, 0)),1,3) "16"
        ,Substr( Sum( Decode( i, 17, 1, 0)),1,3) "17"
        ,Substr( Sum( Decode( i, 18, 1, 0)),1,3) "18"
        ,Substr( Sum( Decode( i, 19, 1, 0)),1,3) "19"
        ,Substr( Sum( Decode( i, 20, 1, 0)),1,3) "20"
        ,Substr( Sum( Decode( i, 21, 1, 0)),1,3) "21"
        ,Substr( Sum( Decode( i, 22, 1, 0)),1,3) "22"
        ,Substr( Sum( Decode( i, 23, 1, 0)),1,3) "23"
        ,Substr( Sum( Decode( i, 24, 1, 0)),1,3) "24"
        ,Substr( Sum( Decode( i, 25, 1, 0)),1,3) "25"
        ,Substr( Sum( Decode( i, 26, 1, 0)),1,3) "26"
        ,Substr( Sum( Decode( i, 27, 1, 0)),1,3) "27"
        ,Substr( Sum( Decode( i, 28, 1, 0)),1,3) "28"
        ,Substr( Sum( Decode( i, 29, 1, 0)),1,3) "29"
        ,Substr( Sum( Decode( i, 30, 1, 0)),1,3) "30"
        ,Substr( Count(i),1,5) "total"
FROM    (SELECT column_value i FROM Table( ssstest_user.fnc_ssstest_expondist( 1/3, 1000, 'pdf3')));



PROMPT *** pdf - 20:
SELECT
         Substr( Sum( Decode( i, 0, 1, 0)),1,3) "0"
        ,Substr( Sum( Decode( i, 1, 1, 0)),1,3) "1"
        ,Substr( Sum( Decode( i, 2, 1, 0)),1,3) "2"
        ,Substr( Sum( Decode( i, 3, 1, 0)),1,3) "3"
        ,Substr( Sum( Decode( i, 4, 1, 0)),1,3) "4"
        ,Substr( Sum( Decode( i, 5, 1, 0)),1,3) "5"
        ,Substr( Sum( Decode( i, 6, 1, 0)),1,3) "6"
        ,Substr( Sum( Decode( i, 7, 1, 0)),1,3) "7"
        ,Substr( Sum( Decode( i, 8, 1, 0)),1,3) "8"
        ,Substr( Sum( Decode( i, 9, 1, 0)),1,3) "9"
        ,Substr( Sum( Decode( i, 10, 1, 0)),1,3) "10"
        ,Substr( Sum( Decode( i, 11, 1, 0)),1,3) "11"
        ,Substr( Sum( Decode( i, 12, 1, 0)),1,3) "12"
        ,Substr( Sum( Decode( i, 13, 1, 0)),1,3) "13"
        ,Substr( Sum( Decode( i, 14, 1, 0)),1,3) "14"
        ,Substr( Sum( Decode( i, 15, 1, 0)),1,3) "15"
        ,Substr( Sum( Decode( i, 16, 1, 0)),1,3) "16"
        ,Substr( Sum( Decode( i, 17, 1, 0)),1,3) "17"
        ,Substr( Sum( Decode( i, 18, 1, 0)),1,3) "18"
        ,Substr( Sum( Decode( i, 19, 1, 0)),1,3) "19"
        ,Substr( Sum( Decode( i, 20, 1, 0)),1,3) "20"
        ,Substr( Sum( Decode( i, 21, 1, 0)),1,3) "21"
        ,Substr( Sum( Decode( i, 22, 1, 0)),1,3) "22"
        ,Substr( Sum( Decode( i, 23, 1, 0)),1,3) "23"
        ,Substr( Sum( Decode( i, 24, 1, 0)),1,3) "24"
        ,Substr( Sum( Decode( i, 25, 1, 0)),1,3) "25"
        ,Substr( Sum( Decode( i, 26, 1, 0)),1,3) "26"
        ,Substr( Sum( Decode( i, 27, 1, 0)),1,3) "27"
        ,Substr( Sum( Decode( i, 28, 1, 0)),1,3) "28"
        ,Substr( Sum( Decode( i, 29, 1, 0)),1,3) "29"
        ,Substr( Sum( Decode( i, 30, 1, 0)),1,3) "30"
        ,Substr( Count(i),1,5) "total"
FROM    (SELECT column_value i FROM Table( ssstest_user.fnc_ssstest_expondist( 1/3, 20, 'pdf')));


PROMPT *** pdf2 - 20:
SELECT
         Substr( Sum( Decode( i, 0, 1, 0)),1,3) "0"
        ,Substr( Sum( Decode( i, 1, 1, 0)),1,3) "1"
        ,Substr( Sum( Decode( i, 2, 1, 0)),1,3) "2"
        ,Substr( Sum( Decode( i, 3, 1, 0)),1,3) "3"
        ,Substr( Sum( Decode( i, 4, 1, 0)),1,3) "4"
        ,Substr( Sum( Decode( i, 5, 1, 0)),1,3) "5"
        ,Substr( Sum( Decode( i, 6, 1, 0)),1,3) "6"
        ,Substr( Sum( Decode( i, 7, 1, 0)),1,3) "7"
        ,Substr( Sum( Decode( i, 8, 1, 0)),1,3) "8"
        ,Substr( Sum( Decode( i, 9, 1, 0)),1,3) "9"
        ,Substr( Sum( Decode( i, 10, 1, 0)),1,3) "10"
        ,Substr( Sum( Decode( i, 11, 1, 0)),1,3) "11"
        ,Substr( Sum( Decode( i, 12, 1, 0)),1,3) "12"
        ,Substr( Sum( Decode( i, 13, 1, 0)),1,3) "13"
        ,Substr( Sum( Decode( i, 14, 1, 0)),1,3) "14"
        ,Substr( Sum( Decode( i, 15, 1, 0)),1,3) "15"
        ,Substr( Sum( Decode( i, 16, 1, 0)),1,3) "16"
        ,Substr( Sum( Decode( i, 17, 1, 0)),1,3) "17"
        ,Substr( Sum( Decode( i, 18, 1, 0)),1,3) "18"
        ,Substr( Sum( Decode( i, 19, 1, 0)),1,3) "19"
        ,Substr( Sum( Decode( i, 20, 1, 0)),1,3) "20"
        ,Substr( Sum( Decode( i, 21, 1, 0)),1,3) "21"
        ,Substr( Sum( Decode( i, 22, 1, 0)),1,3) "22"
        ,Substr( Sum( Decode( i, 23, 1, 0)),1,3) "23"
        ,Substr( Sum( Decode( i, 24, 1, 0)),1,3) "24"
        ,Substr( Sum( Decode( i, 25, 1, 0)),1,3) "25"
        ,Substr( Sum( Decode( i, 26, 1, 0)),1,3) "26"
        ,Substr( Sum( Decode( i, 27, 1, 0)),1,3) "27"
        ,Substr( Sum( Decode( i, 28, 1, 0)),1,3) "28"
        ,Substr( Sum( Decode( i, 29, 1, 0)),1,3) "29"
        ,Substr( Sum( Decode( i, 30, 1, 0)),1,3) "30"
        ,Substr( Count(i),1,5) "total"
FROM    (SELECT column_value i FROM Table( ssstest_user.fnc_ssstest_expondist( 1/3, 20, 'pdf2')));


PROMPT *** pdf3 - 20:
SELECT
         Substr( Sum( Decode( i, 0, 1, 0)),1,3) "0"
        ,Substr( Sum( Decode( i, 1, 1, 0)),1,3) "1"
        ,Substr( Sum( Decode( i, 2, 1, 0)),1,3) "2"
        ,Substr( Sum( Decode( i, 3, 1, 0)),1,3) "3"
        ,Substr( Sum( Decode( i, 4, 1, 0)),1,3) "4"
        ,Substr( Sum( Decode( i, 5, 1, 0)),1,3) "5"
        ,Substr( Sum( Decode( i, 6, 1, 0)),1,3) "6"
        ,Substr( Sum( Decode( i, 7, 1, 0)),1,3) "7"
        ,Substr( Sum( Decode( i, 8, 1, 0)),1,3) "8"
        ,Substr( Sum( Decode( i, 9, 1, 0)),1,3) "9"
        ,Substr( Sum( Decode( i, 10, 1, 0)),1,3) "10"
        ,Substr( Sum( Decode( i, 11, 1, 0)),1,3) "11"
        ,Substr( Sum( Decode( i, 12, 1, 0)),1,3) "12"
        ,Substr( Sum( Decode( i, 13, 1, 0)),1,3) "13"
        ,Substr( Sum( Decode( i, 14, 1, 0)),1,3) "14"
        ,Substr( Sum( Decode( i, 15, 1, 0)),1,3) "15"
        ,Substr( Sum( Decode( i, 16, 1, 0)),1,3) "16"
        ,Substr( Sum( Decode( i, 17, 1, 0)),1,3) "17"
        ,Substr( Sum( Decode( i, 18, 1, 0)),1,3) "18"
        ,Substr( Sum( Decode( i, 19, 1, 0)),1,3) "19"
        ,Substr( Sum( Decode( i, 20, 1, 0)),1,3) "20"
        ,Substr( Sum( Decode( i, 21, 1, 0)),1,3) "21"
        ,Substr( Sum( Decode( i, 22, 1, 0)),1,3) "22"
        ,Substr( Sum( Decode( i, 23, 1, 0)),1,3) "23"
        ,Substr( Sum( Decode( i, 24, 1, 0)),1,3) "24"
        ,Substr( Sum( Decode( i, 25, 1, 0)),1,3) "25"
        ,Substr( Sum( Decode( i, 26, 1, 0)),1,3) "26"
        ,Substr( Sum( Decode( i, 27, 1, 0)),1,3) "27"
        ,Substr( Sum( Decode( i, 28, 1, 0)),1,3) "28"
        ,Substr( Sum( Decode( i, 29, 1, 0)),1,3) "29"
        ,Substr( Sum( Decode( i, 30, 1, 0)),1,3) "30"
        ,Substr( Count(i),1,5) "total"
FROM    (SELECT column_value i FROM Table( ssstest_user.fnc_ssstest_expondist( 1/3, 20, 'pdf3')));

DECLARE
	t_expondists		ssstest_user.ssstest_expondist_ty;
	t_string1		VARCHAR2(4000);
	t_string2		VARCHAR2(4000);
	t_tau			INT;
	t_sum_before_tau	INT := 0;
	t_sum_before_tau_per	NUMBER(5,1);
BEGIN
	t_expondists := ssstest_user.fnc_ssstest_expondist( 1/3, 20, 'pdf');
	t_tau := 60/20;

	dbms_output.put_line(CHR(10) || 'Summary of the exponentially distributed RV - pdf:');
	dbms_output.put_line('0   1   2   3   4   5   6   7   8   9   10  11  12  13  14  15  16  17  18  19  20  21  22  23  24  25  26  27  28  29  30  total');
	dbms_output.put_line('--- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ------');

	SELECT
		 i0 || Rpad( ' ' , 4-length(i0))  ||
		 i1 || Rpad( ' ' , 4-length(i1))  ||
		 i2 || Rpad( ' ' , 4-length(i2))  ||
		 i3 || Rpad( ' ' , 4-length(i3))  ||
		 i4 || Rpad( ' ' , 4-length(i4))  ||
		 i5 || Rpad( ' ' , 4-length(i5))  ||
		 i6 || Rpad( ' ' , 4-length(i6))  ||
		 i7 || Rpad( ' ' , 4-length(i7))  ||
		 i8 || Rpad( ' ' , 4-length(i8))  ||
		 i9 || Rpad( ' ' , 4-length(i9))  ||
		 i10 || Rpad(' ', 4-length(i10)) ||
		 i11 || Rpad(' ', 4-length(i11)) ||
		 i12 || Rpad(' ', 4-length(i12)) ||
		 i13 || Rpad(' ', 4-length(i13)) ||
		 i14 || Rpad(' ', 4-length(i14)) ||
		 i15 || Rpad(' ', 4-length(i15)) ||
		 i16 || Rpad(' ', 4-length(i16)) ||
		 i17 || Rpad(' ', 4-length(i17)) ||
		 i18 || Rpad(' ', 4-length(i18)) ||
		 i19 || Rpad(' ', 4-length(i19)) ||
		 i20 || Rpad(' ', 4-length(i20)) ||
		 i21 || Rpad(' ', 4-length(i21)) ||
		 i22 || Rpad(' ', 4-length(i22)) ||
		 i23 || Rpad(' ', 4-length(i23)) ||
		 i24 || Rpad(' ', 4-length(i24)) ||
		 i25 || Rpad(' ', 4-length(i25)) ||
		 i26 || Rpad(' ', 4-length(i26)) ||
		 i27 || Rpad(' ', 4-length(i27)) ||
		 i28 || Rpad(' ', 4-length(i28)) ||
		 i29 || Rpad(' ', 4-length(i29)) ||
		 i30 || Rpad(' ', 4-length(i30)) ||
		 i31 || Rpad(' ', 4-length(i31)) 
	INTO	t_string1
	FROM (
		SELECT
	       	 Substr( Sum( Decode( column_value, 0, 1, 0)),1,3) i0
       		,Substr( Sum( Decode( column_value, 1, 1, 0)),1,3) i1
       		,Substr( Sum( Decode( column_value, 2, 1, 0)),1,3) i2
	        ,Substr( Sum( Decode( column_value, 3, 1, 0)),1,3) i3
	        ,Substr( Sum( Decode( column_value, 4, 1, 0)),1,3) i4
       		,Substr( Sum( Decode( column_value, 5, 1, 0)),1,3) i5
       		,Substr( Sum( Decode( column_value, 6, 1, 0)),1,3) i6
       		,Substr( Sum( Decode( column_value, 7, 1, 0)),1,3) i7
       		,Substr( Sum( Decode( column_value, 8, 1, 0)),1,3) i8
       		,Substr( Sum( Decode( column_value, 9, 1, 0)),1,3) i9
       		,Substr( Sum( Decode( column_value, 10, 1, 0)),1,3) i10
       		,Substr( Sum( Decode( column_value, 11, 1, 0)),1,3) i11
       		,Substr( Sum( Decode( column_value, 12, 1, 0)),1,3) i12
       		,Substr( Sum( Decode( column_value, 13, 1, 0)),1,3) i13
       		,Substr( Sum( Decode( column_value, 14, 1, 0)),1,3) i14
       		,Substr( Sum( Decode( column_value, 15, 1, 0)),1,3) i15
       		,Substr( Sum( Decode( column_value, 16, 1, 0)),1,3) i16
       		,Substr( Sum( Decode( column_value, 17, 1, 0)),1,3) i17
       		,Substr( Sum( Decode( column_value, 18, 1, 0)),1,3) i18
       		,Substr( Sum( Decode( column_value, 19, 1, 0)),1,3) i19
       		,Substr( Sum( Decode( column_value, 20, 1, 0)),1,3) i20
       		,Substr( Sum( Decode( column_value, 21, 1, 0)),1,3) i21
       		,Substr( Sum( Decode( column_value, 22, 1, 0)),1,3) i22
       		,Substr( Sum( Decode( column_value, 23, 1, 0)),1,3) i23
       		,Substr( Sum( Decode( column_value, 24, 1, 0)),1,3) i24
       		,Substr( Sum( Decode( column_value, 25, 1, 0)),1,3) i25
       		,Substr( Sum( Decode( column_value, 26, 1, 0)),1,3) i26
       		,Substr( Sum( Decode( column_value, 27, 1, 0)),1,3) i27
       		,Substr( Sum( Decode( column_value, 28, 1, 0)),1,3) i28
       		,Substr( Sum( Decode( column_value, 29, 1, 0)),1,3) i29
       		,Substr( Sum( Decode( column_value, 30, 1, 0)),1,3) i30
       		,Substr( Count(column_value),1,5) i31
	FROM    ( Table( t_expondists))
	);
	dbms_output.put_line(t_string1 || CHR(10));

	FOR i IN 1..t_expondists.COUNT LOOP
		t_string2 := t_string2 || t_expondists(i) || ' ';	
		IF ( t_expondists(i) <= t_tau) THEN
			t_sum_before_tau := t_sum_before_tau + 1;
		END IF;
	END LOOP;
	t_sum_before_tau_per := t_sum_before_tau/20*100;
	dbms_output.put_line('Detailed distribution used for the above summary - pdf: Number of interarrival times -le 60/20=3sec - ' || 
				t_sum_before_tau || ' (' || t_sum_before_tau_per || '%)');
	dbms_output.put_line(t_string2 ||', total='||t_expondists.COUNT);
END;
/

DECLARE
	t_expondists		ssstest_user.ssstest_expondist_ty;
	t_string1		VARCHAR2(4000);
	t_string2		VARCHAR2(4000);
BEGIN
	t_expondists := ssstest_user.fnc_ssstest_expondist( 1/3, 20, 'pdf2');

	dbms_output.put_line(CHR(10) || 'Summary of the Poisson distributed RV - pdf2:');
	dbms_output.put_line('0   1   2   3   4   5   6   7   8   9   10  11  12  13  14  15  16  17  18  19  20  21  22  23  24  25  26  27  28  29  30  total');
	dbms_output.put_line('--- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ------');

	SELECT
		 i0 || Rpad( ' ' , 4-length(i0))  ||
		 i1 || Rpad( ' ' , 4-length(i1))  ||
		 i2 || Rpad( ' ' , 4-length(i2))  ||
		 i3 || Rpad( ' ' , 4-length(i3))  ||
		 i4 || Rpad( ' ' , 4-length(i4))  ||
		 i5 || Rpad( ' ' , 4-length(i5))  ||
		 i6 || Rpad( ' ' , 4-length(i6))  ||
		 i7 || Rpad( ' ' , 4-length(i7))  ||
		 i8 || Rpad( ' ' , 4-length(i8))  ||
		 i9 || Rpad( ' ' , 4-length(i9))  ||
		 i10 || Rpad(' ', 4-length(i10)) ||
		 i11 || Rpad(' ', 4-length(i11)) ||
		 i12 || Rpad(' ', 4-length(i12)) ||
		 i13 || Rpad(' ', 4-length(i13)) ||
		 i14 || Rpad(' ', 4-length(i14)) ||
		 i15 || Rpad(' ', 4-length(i15)) ||
		 i16 || Rpad(' ', 4-length(i16)) ||
		 i17 || Rpad(' ', 4-length(i17)) ||
		 i18 || Rpad(' ', 4-length(i18)) ||
		 i19 || Rpad(' ', 4-length(i19)) ||
		 i20 || Rpad(' ', 4-length(i20)) ||
		 i21 || Rpad(' ', 4-length(i21)) ||
		 i22 || Rpad(' ', 4-length(i22)) ||
		 i23 || Rpad(' ', 4-length(i23)) ||
		 i24 || Rpad(' ', 4-length(i24)) ||
		 i25 || Rpad(' ', 4-length(i25)) ||
		 i26 || Rpad(' ', 4-length(i26)) ||
		 i27 || Rpad(' ', 4-length(i27)) ||
		 i28 || Rpad(' ', 4-length(i28)) ||
		 i29 || Rpad(' ', 4-length(i29)) ||
		 i30 || Rpad(' ', 4-length(i30)) ||
		 i31 || Rpad(' ', 4-length(i31)) 
	INTO	t_string1
	FROM (
		SELECT
	       	 Substr( Sum( Decode( column_value, 0, 1, 0)),1,3) i0
       		,Substr( Sum( Decode( column_value, 1, 1, 0)),1,3) i1
       		,Substr( Sum( Decode( column_value, 2, 1, 0)),1,3) i2
	        ,Substr( Sum( Decode( column_value, 3, 1, 0)),1,3) i3
	        ,Substr( Sum( Decode( column_value, 4, 1, 0)),1,3) i4
       		,Substr( Sum( Decode( column_value, 5, 1, 0)),1,3) i5
       		,Substr( Sum( Decode( column_value, 6, 1, 0)),1,3) i6
       		,Substr( Sum( Decode( column_value, 7, 1, 0)),1,3) i7
       		,Substr( Sum( Decode( column_value, 8, 1, 0)),1,3) i8
       		,Substr( Sum( Decode( column_value, 9, 1, 0)),1,3) i9
       		,Substr( Sum( Decode( column_value, 10, 1, 0)),1,3) i10
       		,Substr( Sum( Decode( column_value, 11, 1, 0)),1,3) i11
       		,Substr( Sum( Decode( column_value, 12, 1, 0)),1,3) i12
       		,Substr( Sum( Decode( column_value, 13, 1, 0)),1,3) i13
       		,Substr( Sum( Decode( column_value, 14, 1, 0)),1,3) i14
       		,Substr( Sum( Decode( column_value, 15, 1, 0)),1,3) i15
       		,Substr( Sum( Decode( column_value, 16, 1, 0)),1,3) i16
       		,Substr( Sum( Decode( column_value, 17, 1, 0)),1,3) i17
       		,Substr( Sum( Decode( column_value, 18, 1, 0)),1,3) i18
       		,Substr( Sum( Decode( column_value, 19, 1, 0)),1,3) i19
       		,Substr( Sum( Decode( column_value, 20, 1, 0)),1,3) i20
       		,Substr( Sum( Decode( column_value, 21, 1, 0)),1,3) i21
       		,Substr( Sum( Decode( column_value, 22, 1, 0)),1,3) i22
       		,Substr( Sum( Decode( column_value, 23, 1, 0)),1,3) i23
       		,Substr( Sum( Decode( column_value, 24, 1, 0)),1,3) i24
       		,Substr( Sum( Decode( column_value, 25, 1, 0)),1,3) i25
       		,Substr( Sum( Decode( column_value, 26, 1, 0)),1,3) i26
       		,Substr( Sum( Decode( column_value, 27, 1, 0)),1,3) i27
       		,Substr( Sum( Decode( column_value, 28, 1, 0)),1,3) i28
       		,Substr( Sum( Decode( column_value, 29, 1, 0)),1,3) i29
       		,Substr( Sum( Decode( column_value, 30, 1, 0)),1,3) i30
       		--,Substr( Count(column_value),1,5) - Sum( Decode( column_value, 0, 1, 0)) i31
       		--,Substr( Count(column_value) - Sum( Decode( column_value, 0, 1, 0)), 1,5) i31
       		,Substr( Count(column_value), 1,5) i31
	FROM    ( Table( t_expondists))
	);
	dbms_output.put_line(t_string1 || CHR(10));

	FOR i IN 1..t_expondists.COUNT LOOP
		t_string2 := t_string2 || t_expondists(i) || ' ';	
	END LOOP;
	dbms_output.put_line('Detailed distribution (used for the above summary) - pdf2:');
	dbms_output.put_line(t_string2 ||', total='||t_expondists.COUNT);
END;
/


DECLARE
	t_expondists		ssstest_user.ssstest_expondist_ty;
	t_string1		VARCHAR2(4000);
	t_string2		VARCHAR2(4000);
BEGIN
	t_expondists := ssstest_user.fnc_ssstest_expondist( 1/3, 20, 'pdf3');

	dbms_output.put_line(CHR(10) || 'Summary of the Poisson distributed RV - pdf3:');
	dbms_output.put_line('0   1   2   3   4   5   6   7   8   9   10  11  12  13  14  15  16  17  18  19  20  21  22  23  24  25  26  27  28  29  30  total');
	dbms_output.put_line('--- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- ------');

	SELECT
		 i0 || Rpad( ' ' , 4-length(i0))  ||
		 i1 || Rpad( ' ' , 4-length(i1))  ||
		 i2 || Rpad( ' ' , 4-length(i2))  ||
		 i3 || Rpad( ' ' , 4-length(i3))  ||
		 i4 || Rpad( ' ' , 4-length(i4))  ||
		 i5 || Rpad( ' ' , 4-length(i5))  ||
		 i6 || Rpad( ' ' , 4-length(i6))  ||
		 i7 || Rpad( ' ' , 4-length(i7))  ||
		 i8 || Rpad( ' ' , 4-length(i8))  ||
		 i9 || Rpad( ' ' , 4-length(i9))  ||
		 i10 || Rpad(' ', 4-length(i10)) ||
		 i11 || Rpad(' ', 4-length(i11)) ||
		 i12 || Rpad(' ', 4-length(i12)) ||
		 i13 || Rpad(' ', 4-length(i13)) ||
		 i14 || Rpad(' ', 4-length(i14)) ||
		 i15 || Rpad(' ', 4-length(i15)) ||
		 i16 || Rpad(' ', 4-length(i16)) ||
		 i17 || Rpad(' ', 4-length(i17)) ||
		 i18 || Rpad(' ', 4-length(i18)) ||
		 i19 || Rpad(' ', 4-length(i19)) ||
		 i20 || Rpad(' ', 4-length(i20)) ||
		 i21 || Rpad(' ', 4-length(i21)) ||
		 i22 || Rpad(' ', 4-length(i22)) ||
		 i23 || Rpad(' ', 4-length(i23)) ||
		 i24 || Rpad(' ', 4-length(i24)) ||
		 i25 || Rpad(' ', 4-length(i25)) ||
		 i26 || Rpad(' ', 4-length(i26)) ||
		 i27 || Rpad(' ', 4-length(i27)) ||
		 i28 || Rpad(' ', 4-length(i28)) ||
		 i29 || Rpad(' ', 4-length(i29)) ||
		 i30 || Rpad(' ', 4-length(i30)) ||
		 i31 || Rpad(' ', 4-length(i31)) 
	INTO	t_string1
	FROM (
		SELECT
	       	 Substr( Sum( Decode( column_value, 0, 1, 0)),1,3) i0
       		,Substr( Sum( Decode( column_value, 1, 1, 0)),1,3) i1
       		,Substr( Sum( Decode( column_value, 2, 1, 0)),1,3) i2
	        ,Substr( Sum( Decode( column_value, 3, 1, 0)),1,3) i3
	        ,Substr( Sum( Decode( column_value, 4, 1, 0)),1,3) i4
       		,Substr( Sum( Decode( column_value, 5, 1, 0)),1,3) i5
       		,Substr( Sum( Decode( column_value, 6, 1, 0)),1,3) i6
       		,Substr( Sum( Decode( column_value, 7, 1, 0)),1,3) i7
       		,Substr( Sum( Decode( column_value, 8, 1, 0)),1,3) i8
       		,Substr( Sum( Decode( column_value, 9, 1, 0)),1,3) i9
       		,Substr( Sum( Decode( column_value, 10, 1, 0)),1,3) i10
       		,Substr( Sum( Decode( column_value, 11, 1, 0)),1,3) i11
       		,Substr( Sum( Decode( column_value, 12, 1, 0)),1,3) i12
       		,Substr( Sum( Decode( column_value, 13, 1, 0)),1,3) i13
       		,Substr( Sum( Decode( column_value, 14, 1, 0)),1,3) i14
       		,Substr( Sum( Decode( column_value, 15, 1, 0)),1,3) i15
       		,Substr( Sum( Decode( column_value, 16, 1, 0)),1,3) i16
       		,Substr( Sum( Decode( column_value, 17, 1, 0)),1,3) i17
       		,Substr( Sum( Decode( column_value, 18, 1, 0)),1,3) i18
       		,Substr( Sum( Decode( column_value, 19, 1, 0)),1,3) i19
       		,Substr( Sum( Decode( column_value, 20, 1, 0)),1,3) i20
       		,Substr( Sum( Decode( column_value, 21, 1, 0)),1,3) i21
       		,Substr( Sum( Decode( column_value, 22, 1, 0)),1,3) i22
       		,Substr( Sum( Decode( column_value, 23, 1, 0)),1,3) i23
       		,Substr( Sum( Decode( column_value, 24, 1, 0)),1,3) i24
       		,Substr( Sum( Decode( column_value, 25, 1, 0)),1,3) i25
       		,Substr( Sum( Decode( column_value, 26, 1, 0)),1,3) i26
       		,Substr( Sum( Decode( column_value, 27, 1, 0)),1,3) i27
       		,Substr( Sum( Decode( column_value, 28, 1, 0)),1,3) i28
       		,Substr( Sum( Decode( column_value, 29, 1, 0)),1,3) i29
       		,Substr( Sum( Decode( column_value, 30, 1, 0)),1,3) i30
       		--,Substr( Count(column_value),1,5) - Sum( Decode( column_value, 0, 1, 0)) i31
       		--,Substr( Count(column_value) - Sum( Decode( column_value, 0, 1, 0)), 1,5) i31
       		,Substr( Count(column_value), 1,5) i31
	FROM    ( Table( t_expondists))
	);
	dbms_output.put_line(t_string1 || CHR(10));

	FOR i IN 1..t_expondists.COUNT LOOP
		t_string2 := t_string2 || t_expondists(i) || ' ';	
	END LOOP;
	dbms_output.put_line('Detailed distribution (used for the above summary) - pdf3:');
	dbms_output.put_line(t_string2 ||', total='||t_expondists.COUNT);
END;
/


set doc off
/**********************************************
DECLARE
	t_exp			INT;
	t_min_product		NUMBER;
	t_limit			NUMBER;
	i			INT;
	j			INT;
   p_lambda NUMBER := 1/60/20;
   p_return_array_size INT := 20;
   t_exp_string VARCHAR2(2000);
BEGIN
               t_limit := exp(-1*p_lambda);
                FOR i IN 1..p_return_array_size LOOP
                        t_min_product := 1; j := 0;
                        WHILE ( t_min_product >= t_limit) LOOP
                                t_min_product := t_min_product * dbms_random.value;
                                j := j + 1;
                        END LOOP;
                        -- raise_application_error( -20101, 'i='||i||',t_min_product='||t_min_product||', t_limit='||t_limit);
                        t_exp := j-1;
			t_exp_string := t_exp_string || ' ' || t_exp;
                END LOOP;
		dbms_output.put_line( t_exp_string );
END;
/
**********************************************/
