
-- It's possible in theory that the tablespace configuration settings
-- (extent management, segment space management, default storage parameters) matter, so here's mine:
-- adjust the datafile clause to your system:
/*******************
create tablespace ssstest_tbs 
datafile '/oracle/oradata/dani11/ssstest_tbs_01.dbf' size 10m autoextend on next 1m
blocksize 8k
extent management local --dictionary 
-- uniform 64k
-- default storage (initial 40k next 40k pctincrease 50)
-- segment space managment manual
/
-- if you go with ssstest_tbs, than just use global search/replace :1,$s/users/ssstest_tbs/g
*******************/
set lin 150 pages 100 feed on ver off
col plan_plus_exp form a100

spool ssstest_setup_main.log

-- Schema owner of the scale test DB objects:
drop user ssstest_user cascade;
grant connect, resource to ssstest_user identified by ssstest_user;
alter user ssstest_user default tablespace users;
alter user ssstest_user temporary tablespace temp;

-- User used to run the test:
-- drop user scott cascade;
grant create session, alter session to scott identified by tiger;
alter user scott default tablespace users;
alter user scott temporary tablespace temp;

-- for the autotrace: either use an existing plan table or login as ssstest_user and create one (?/rdbms/admin/utlxplan):
@?/rdbms/admin/utlxplan
create or replace synonym ssstest_user.plan_table for plan_table;
grant select,insert,update,delete on plan_table to ssstest_user;

-- Run as sys
GRANT SELECT ON sys.v_$mystat TO ssstest_user;
GRANT SELECT ON sys.v_$statname TO ssstest_user;
GRANT SELECT ON sys.v_$latch TO ssstest_user;

-- Cleanup is done by recreating the schema, so the following needed only for manual fixing:
-- drop table ssstest_user.stest_runstats;
-- drop table stest_uesr.stest_runstats_messages;
-- drop table ssstest_user.stest_runstats_service_time;

CREATE TABLE ssstest_user.ssstest_runstats(
	runid		VARCHAR2(15),
	iteration_id	INT,
	name		VARCHAR2(80),
	value		INT
);

GRANT ALL ON ssstest_user.ssstest_runstats TO PUBLIC;

CREATE OR REPLACE VIEW ssstest_user.v_stats AS
	SELECT	'STAT....'||a.name name, b.value
	FROM	v$statname a, v$mystat b
	WHERE	a.statistic# = b.statistic#
UNION ALL
	SELECT	'LATCH...'||name, gets
	FROM	v$latch;

GRANT EXECUTE ON dbms_lock TO scott;

CREATE OR REPLACE TYPE ssstest_user.ot_varray_rampup_users AS VARRAY(11) OF INT
/

GRANT EXECUTE ON ssstest_user.ot_varray_rampup_users TO scott
/

CREATE TABLE ssstest_user.ssstest_runstats_service_time(
	run_id		INT,
	flg_begin_end	INT,
	timing		INT
);

CREATE OR REPLACE TYPE ssstest_user.ot_varray_sleep_times AS VARRAY(20) OF INT
/

GRANT EXECUTE ON ssstest_user.ot_varray_sleep_times TO scott
/

CREATE TABLE ssstest_user.ssstest_runstats_messages(
        run_id          INT,
        iteration_id    INT,
        user_id         INT,
        concurrent_users INT,                   -- not relational, but saves another table :-)
        create_user     VARCHAR2(30)    DEFAULT user,
        create_datetime TIMESTAMP       DEFAULT localtimestamp,
--      create_datetime DATE            DEFAULT sysdate,
        CONSTRAINT ssstest_runstats_messages_pk PRIMARY KEY(run_id, iteration_id, user_id)
)
/

GRANT SELECT,INSERT,DELETE ON ssstest_user.ssstest_runstats_messages TO public
/

-- statefull extra tables:
CREATE TABLE ssstest_user.ssstest_login(
	run_id		INT,
	iteration_id	INT,
	user_id		INT,
        create_user     VARCHAR2(30)    DEFAULT user,
        create_datetime TIMESTAMP       DEFAULT localtimestamp,
	CONSTRAINT ssstest_login_pk PRIMARY KEY( run_id, iteration_id, user_id)
)
/

GRANT SELECT,INSERT,DELETE ON ssstest_user.ssstest_login TO public
/

-------------
@ssstest_setup_expondist.sql
-------------
@ssstest_setup_runstats_pkg.sql
-------------
@ssstest_setup_scenario.sql
-------------
@ssstest_setup_aq.sql

spool off

