-- Modified Tom Kytes's test-harness to test 1) Elapsed time diff 2) Amount of resources required:
set doc off
DOC
/**************
-- # SSS-test
-- # Dali Database Solutions Inc. (C) Copyright. All rights reserved.
-- # http://www.tns-0.com

Run scaletest_setup.sql first
**************/
#

set serveroutput on def off

CREATE OR REPLACE PACKAGE ssstest_user.pkg_ssstest_runstats IS
  PROCEDURE prc_snap (
	 p_runid	INTEGER 	DEFAULT 0
	,p_iteration_id	INTEGER
	,p_reset 	BOOLEAN 	DEFAULT false
  );

  PROCEDURE prc_record_stats( 
	 p_title 	VARCHAR2
	,p_iteration_id	INTEGER
	,p_ela_time 	NUMBER		DEFAULT 0
  );

  PROCEDURE prc_results(
	 p_distribution	CHAR		DEFAULT 'Z'
	,p_rampup_users ot_varray_rampup_users
	,p_sleep_times	ot_varray_sleep_times
	,p_report	CHAR	-- can be 'RES', 'ELA', 'ALL' or 'DBG'
  );

  PROCEDURE prc_service_time_snap (
	 p_run_id	INTEGER
	,p_flg_begin_end INTEGER
  );

  PROCEDURE prc_service_time_results (
	 p_runs		INTEGER
  );

  PROCEDURE prc_draw_line;	-- show probably made private

END pkg_ssstest_runstats;
/

sho err


CREATE OR REPLACE PACKAGE BODY ssstest_user.pkg_ssstest_runstats IS
  g_start		NUMBER;		-- works only within one session

  PROCEDURE prc_record_stats(
	 p_title 	VARCHAR2
	,p_iteration_id	INTEGER
	,p_ela_time 	NUMBER		DEFAULT 0
  ) AS
  PRAGMA AUTONOMOUS_TRANSACTION;
  BEGIN
	INSERT INTO ssstest_user.ssstest_runstats 
		SELECT p_title, p_iteration_id, v_stats.* FROM v_stats
		UNION ALL
		SELECT p_title, p_iteration_id, '... Elapsed Time: Snap  [cs]', p_ela_time FROM dual;
	COMMIT;
  END prc_record_stats;

  PROCEDURE prc_snap (
	 p_runid	INTEGER 	DEFAULT 0
	,p_iteration_id	INTEGER
	,p_reset 	BOOLEAN 	DEFAULT false
  ) AS
--	t_start		INTEGER;

	TYPE rowid_ty IS TABLE OF rowid INDEX BY BINARY_INTEGER;
	l_rowids	rowid_ty;
	TYPE runstats_runid_ty IS TABLE OF ssstest_user.ssstest_runstats.runid%TYPE;
	TYPE runstats_name_ty IS TABLE OF ssstest_user.ssstest_runstats.name%TYPE;
	TYPE runstats_value_ty IS TABLE OF ssstest_user.ssstest_runstats.value%TYPE;
	runstats_runids		runstats_runid_ty;	
	runstats_names		runstats_name_ty;	
	runstats_values 	runstats_value_ty;	
  BEGIN
	IF (p_reset) THEN
		DELETE ssstest_user.ssstest_runstats;
		COMMIT;
		SELECT runid, name, value
		BULK COLLECT INTO runstats_runids, runstats_names, runstats_values
		FROM ssstest_user.ssstest_runstats;	-- to ensure delayed block cleanout doesn't distort 
		prc_record_stats('snap 0', p_iteration_id, dbms_utility.get_time);
--		g_start := dbms_utility.get_time;
	ELSE
--		ROLLBACK; 	-- to avoid delayed block clean out or anythings that might distort
				-- the result because of the previous run

--		prc_record_stats('snap '|| p_runid, (dbms_utility.get_time - g_start));
		prc_record_stats('snap '|| p_runid, p_iteration_id, dbms_utility.get_time);
	END IF;
  END prc_snap;

  PROCEDURE prc_results (
         p_distribution CHAR            DEFAULT 'Z'
        ,p_rampup_users ot_varray_rampup_users
        ,p_sleep_times	ot_varray_sleep_times
	,p_report	CHAR	-- can be 'RES', 'ELA', 'ALL' or 'DBG'
  ) AS
	t_runs			INT;
        t_distro_string         VARCHAR2(50);
	t_elapsed_time_snap	VARCHAR2(2000);
	t_elapsed_time_ulast	VARCHAR2(2000);
	t_elapsed_time_flast	VARCHAR2(2000);
  BEGIN
	SELECT Count( Distinct runid)-1 INTO t_runs FROM ssstest_user.ssstest_runstats;
	IF( t_runs = 2) THEN
		dbms_output.put_line( '*** Error: Not implemented in this version');
	ELSIF( t_runs = 3) THEN
		dbms_output.put_line( '*** Error: Not implemented in this version');
	ELSIF( t_runs = 4) THEN
		dbms_output.put_line( '*** Error: Not implemented in this version');
	ELSIF( t_runs = 5) THEN
		dbms_output.put_line( '*** Error: Not implemented in this version');
	ELSIF( t_runs = 10) THEN
                IF (p_distribution = 'D') THEN
                 	t_distro_string := 'Deterministic Distribution';
                ELSIF (p_distribution = 'Z') THEN
                 	t_distro_string := 'Zero (all-at-once) Distribution';
                ELSIF (p_distribution = 'M') THEN
                 	t_distro_string := 'Exponential Distribution';
                ELSE
                 	t_distro_string := '???????????  Distribution';
                END IF;
                dbms_output.put_line( lpad( t_distro_string, 103));

	     	IF( p_report != 'ELA') THEN	-- meaning it's 'RES', 'ALL' or 'DBG'
		   /********************************** Works for ITERATIONs, get rid of when the other works for ITERATIONS as well
	     	   IF( p_report = 'DBG') THEN
		      FOR r IN (
			SELECT	 Rpad( a.name, 30) ||
				 To_Char( b.value - a.value, '999,999,999') ||
				 To_Char( c.value - b.value, '999,999,999') ||
				 To_Char( d.value - c.value, '999,999,999') ||
				 To_Char( e.value - d.value, '999,999,999') ||
				 To_Char( f.value - e.value, '999,999,999') ||
				 To_Char( g.value - f.value, '999,999,999') ||
				 To_Char( h.value - g.value, '999,999,999') ||
				 To_Char( i.value - h.value, '999,999,999') ||
				 To_Char( j.value - i.value, '999,999,999') ||
				 To_Char( k.value - j.value, '999,999,999') data
			FROM	(SELECT  name
					,Avg(value)	value
				FROM	ssstest_user.ssstest_runstats
				WHERE	runid = 'snap 0'
				GROUP BY runid, name
				) a,
				(SELECT  name
					,Avg(value)	value
				FROM	ssstest_user.ssstest_runstats
				WHERE	runid = 'snap 1'
				GROUP BY runid, name
				) b,
				(SELECT  name
					,Avg(value)	value
				FROM	ssstest_user.ssstest_runstats
				WHERE	runid = 'snap 2'
				GROUP BY runid, name
				) c,
				(SELECT  name
					,Avg(value)	value
				FROM	ssstest_user.ssstest_runstats
				WHERE	runid = 'snap 3'
				GROUP BY runid, name
				) d,
				(SELECT  name
					,Avg(value)	value
				FROM	ssstest_user.ssstest_runstats
				WHERE	runid = 'snap 4'
				GROUP BY runid, name
				) e,
				(SELECT  name
					,Avg(value)	value
				FROM	ssstest_user.ssstest_runstats
				WHERE	runid = 'snap 5'
				GROUP BY runid, name
				) f,
				(SELECT  name
					,Avg(value)	value
				FROM	ssstest_user.ssstest_runstats
				WHERE	runid = 'snap 6'
				GROUP BY runid, name
				) g,
				(SELECT  name
					,Avg(value)	value
				FROM	ssstest_user.ssstest_runstats
				WHERE	runid = 'snap 7'
				GROUP BY runid, name
				) h,
				(SELECT  name
					,Avg(value)	value
				FROM	ssstest_user.ssstest_runstats
				WHERE	runid = 'snap 8'
				GROUP BY runid, name
				) i,
				(SELECT  name
					,Avg(value)	value
				FROM	ssstest_user.ssstest_runstats
				WHERE	runid = 'snap 9'
				GROUP BY runid, name
				) j,
				(SELECT  name
					,Avg(value)	value
				FROM	ssstest_user.ssstest_runstats
				WHERE	runid = 'snap 10'
				GROUP BY runid, name
				) k
			WHERE	a.name = b.name 
			    AND a.name = c.name 
			    AND a.name = d.name 
			    AND a.name = e.name 
			    AND a.name = f.name 
			    AND a.name = g.name 
			    AND a.name = h.name 
			    AND a.name = i.name 
			    AND a.name = j.name 
			    AND a.name = k.name 
			    AND a.name != '... Elapsed Time: Snap  [cs]'
			    AND (c.value - a.value) > 0		-- what for?
			    AND (c.value - b.value) != (b.value - a.value)
			ORDER BY Abs( (c.value - b.value) - (b.value - a.value) )
		      ) LOOP
			dbms_output.put_line( Substr(r.data, 1, 255));
		      END LOOP;
		   END IF;
		   *******************************************************/


		   FOR r IN (
		     SELECT data
		     FROM  (
                       SELECT
                         Rpad( name, 30) ||
                         To_Char( Sum( Decode( run_id_num, 1, snap_value, 0)),   '999,999,999') ||
                         To_Char( Sum( Decode( run_id_num, 2, snap_value, 0)),   '999,999,999') ||
                         To_Char( Sum( Decode( run_id_num, 3, snap_value, 0)),   '999,999,999') ||
                         To_Char( Sum( Decode( run_id_num, 4, snap_value, 0)),   '999,999,999') ||
                         To_Char( Sum( Decode( run_id_num, 5, snap_value, 0)),   '999,999,999') ||
                         To_Char( Sum( Decode( run_id_num, 6, snap_value, 0)),   '999,999,999') ||
                         To_Char( Sum( Decode( run_id_num, 7, snap_value, 0)),   '999,999,999') ||
                         To_Char( Sum( Decode( run_id_num, 8, snap_value, 0)),   '999,999,999') ||
                         To_Char( Sum( Decode( run_id_num, 9, snap_value, 0)),   '999,999,999') ||
                         To_Char( Sum( Decode( run_id_num,10, snap_value, 0)),   '999,999,999') data,
			 Sum( Decode( run_id_num, 1, snap_value, 0)) snap_value1
                       FROM (
                        SELECT   run_id_num
				,name
				--,value
                                ,value - Lag( value, 1) OVER( PARTITION BY name ORDER BY run_id_num, name) snap_value
                        FROM    (
                                SELECT  runid, To_Number( Substr( runid, 5)) run_id_num,  name, value
                                FROM    ssstest_user.ssstest_runstats
                                WHERE   name != '... Elapsed Time: Snap  [cs]'
				--  and name in ('LATCH...library cache', 'LATCH...enqueues')
                        )
                        ORDER BY run_id_num
                       )
                       WHERE snap_value IS NOT NULL
		       GROUP BY name
		     )
		     WHERE snap_value1 > 0
		     ORDER BY snap_value1
		   ) LOOP
			dbms_output.put_line( Substr(r.data, 1, 255));
		   END LOOP;

		   prc_draw_line;

		   -- /********************************** Works for ITERATIONs, get rid of when the other works for ITERATIONS as well
	     	   IF( p_report = 'DBG') THEN
		      SELECT	 
			 -- Rpad( a.name, 30) ||
			 Rpad( '.d. Elapsed Time: Snap  [cs]', 30) ||
			 To_Char( b.value - a.value, '999,999,999') ||
			 To_Char( c.value - b.value, '999,999,999') ||
			 To_Char( d.value - c.value, '999,999,999') ||
			 To_Char( e.value - d.value, '999,999,999') ||
			 To_Char( f.value - e.value, '999,999,999') ||
			 To_Char( g.value - f.value, '999,999,999') ||
			 To_Char( h.value - g.value, '999,999,999') ||
			 To_Char( i.value - h.value, '999,999,999') ||
			 To_Char( j.value - i.value, '999,999,999') ||
			 To_Char( k.value - j.value, '999,999,999') 
		      INTO t_elapsed_time_snap
		      FROM
			(SELECT  name
				,Avg(value)	value
			FROM	ssstest_user.ssstest_runstats
			WHERE	runid = 'snap 0'
			GROUP BY runid, name
			) a,
			(SELECT  name
				,Avg(value)	value
			FROM	ssstest_user.ssstest_runstats
			WHERE	runid = 'snap 1'
			GROUP BY runid, name
			) b,
			(SELECT  name
				,Avg(value)	value
			FROM	ssstest_user.ssstest_runstats
			WHERE	runid = 'snap 2'
			GROUP BY runid, name
			) c,
			(SELECT  name
				,Avg(value)	value
			FROM	ssstest_user.ssstest_runstats
			WHERE	runid = 'snap 3'
			GROUP BY runid, name
			) d,
			(SELECT  name
				,Avg(value)	value
			FROM	ssstest_user.ssstest_runstats
			WHERE	runid = 'snap 4'
			GROUP BY runid, name
			) e,
			(SELECT  name
				,Avg(value)	value
			FROM	ssstest_user.ssstest_runstats
			WHERE	runid = 'snap 5'
			GROUP BY runid, name
			) f,
			(SELECT  name
				,Avg(value)	value
			FROM	ssstest_user.ssstest_runstats
			WHERE	runid = 'snap 6'
			GROUP BY runid, name
			) g,
			(SELECT  name
				,Avg(value)	value
			FROM	ssstest_user.ssstest_runstats
			WHERE	runid = 'snap 7'
			GROUP BY runid, name
			) h,
			(SELECT  name
				,Avg(value)	value
			FROM	ssstest_user.ssstest_runstats
			WHERE	runid = 'snap 8'
			GROUP BY runid, name
			) i,
			(SELECT  name
				,Avg(value)	value
			FROM	ssstest_user.ssstest_runstats
			WHERE	runid = 'snap 9'
			GROUP BY runid, name
			) j,
			(SELECT  name
				,Avg(value)	value
			FROM	ssstest_user.ssstest_runstats
			WHERE	runid = 'snap 10'
			GROUP BY runid, name
			) k
		      WHERE	
			a.name = b.name 
		    AND a.name = c.name 
		    AND a.name = d.name 
		    AND a.name = e.name 
		    AND a.name = f.name 
		    AND a.name = g.name 
		    AND a.name = h.name 
		    AND a.name = i.name 
		    AND a.name = j.name 
		    AND a.name = k.name 
		    AND a.name = '... Elapsed Time: Snap  [cs]';
		      dbms_output.put_line( t_elapsed_time_snap);
		   END IF;
		   -- *******************************************************************/

		   SELECT	 
			 Rpad( '... Elapsed Time: Snap  [cs]', 30) ||
			 To_Char( Sum( Decode( run_id_num, 1, snap_time, 0)),	'999,999,999') ||
			 To_Char( Sum( Decode( run_id_num, 2, snap_time, 0)),	'999,999,999') ||
			 To_Char( Sum( Decode( run_id_num, 3, snap_time, 0)),	'999,999,999') ||
			 To_Char( Sum( Decode( run_id_num, 4, snap_time, 0)),	'999,999,999') ||
			 To_Char( Sum( Decode( run_id_num, 5, snap_time, 0)),	'999,999,999') ||
			 To_Char( Sum( Decode( run_id_num, 6, snap_time, 0)),	'999,999,999') ||
			 To_Char( Sum( Decode( run_id_num, 7, snap_time, 0)),	'999,999,999') ||
			 To_Char( Sum( Decode( run_id_num, 8, snap_time, 0)),	'999,999,999') ||
			 To_Char( Sum( Decode( run_id_num, 9, snap_time, 0)),	'999,999,999') ||
			 To_Char( Sum( Decode( run_id_num,10, snap_time, 0)),	'999,999,999') 
		   INTO t_elapsed_time_snap
		   FROM	(
			SELECT	 run_id_num
				,value - Lag( value, 1) OVER( ORDER BY run_id_num) snap_time
			FROM	(
				SELECT  runid, To_Number( Substr( runid, 5)) run_id_num,  value
				FROM	ssstest_user.ssstest_runstats
				WHERE	name = '... Elapsed Time: Snap  [cs]'
			)
		   	ORDER BY run_id_num
		   )
		   WHERE snap_time IS NOT NULL;
		   dbms_output.put_line( t_elapsed_time_snap);

		END IF; -- end of 'print resource utilization' decision

	     	IF( p_report = 'DBG') THEN
		   SELECT	
			Rpad( '.d. Elapsed Time: LUser [cs]', 30) ||
			To_Char( Sum( Decode( run_id, 1, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id, 2, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id, 3, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id, 4, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id, 5, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id, 6, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id, 7, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id, 8, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id, 9, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id,10, ela, 0)),		'999,999,999')
		   INTO	t_elapsed_time_ulast
		   FROM	(
			SELECT *
			FROM	(
				SELECT   
					 rm1.run_id
				--      ,rm1.iteration_id
				        ,rm1.user_id
				--	,rm1.create_datetime
		       		 	,rm1.concurrent_users
				--
				      	,((To_Char( rm1.create_datetime, 'SSSSS') - 
					   To_Char( rm2.create_datetime, 'SSSSS')
					  ) + 
					  (To_Char( rm1.create_datetime, 'FF2') - 
					   To_Char( rm2.create_datetime, 'FF2')
					  )/100
					 )*100 ela
				FROM    ( 
					SELECT * 
					FROM 	(
						SELECT   rm1.run_id
	  					--      ,rm1.iteration_id
						        ,rm1.user_id
						        ,rm1.create_datetime
					    	        ,rm1.concurrent_users
					        --
					                ,Max( rm1.create_datetime) OVER( PARTITION BY rm1.run_id ORDER BY rm1.run_id) max_datetime
					  	FROM     ssstest_user.ssstest_runstats_messages rm1 
						WHERE   rm1.user_id != 0
						)
					WHERE	create_datetime = max_datetime
					) rm1 					/* current */
				        ,ssstest_user.ssstest_runstats_messages rm2 /* previous */
				WHERE   rm2.user_id = 0
		       		    AND rm2.run_id = rm1.run_id-1
				ORDER BY rm1.run_id ASC
				)
		   );
		   dbms_output.put_line( t_elapsed_time_ulast);

		   -- If "All-At-Once (no) distribution (aka 'Z'), than substract 0, i.e. do nothing
		   -- Else (Linear or Exponential distribution, than substact pure sleep time:
		   --   For Linear/Deterministic distribution:
		   --   	Sleep time = Duration/(Users in the run)*(Users-1), e.g. for 5 users run/ramp-up in 60sec interval
		   --   	sleep time = 60/5*4=12sec*4=48sec or 4800 csec
		   --   	(By convention, Duration is recorded as the last element of p_rampup_users collection)
		   --   For Exponential distribution: 
		   --	can't be predicted precisely (based on generation of random numbers), but...
		   --   	can be measured, so pass as a parameter 1-st/last user sleep times

		   --select p_rampup_users(1) into t_elapsed_time_ulast from dual;
		   --dbms_output.put_line( t_elapsed_time_ulast);
		   --select p_rampup_users(5) into t_elapsed_time_ulast from dual;
		   --dbms_output.put_line( t_elapsed_time_ulast);
		   --select to_number(p_rampup_users(4))-1 into t_elapsed_time_ulast from dual;
		   --dbms_output.put_line( t_elapsed_time_ulast);
		   --select to_number(p_rampup_users(5))-1 into t_elapsed_time_ulast from dual;
		   --dbms_output.put_line( t_elapsed_time_ulast);

		   IF ( p_distribution != 'Z') THEN
			select  rpad('.d. sleep time (distr='||p_distribution||')......', 30) ||
			 	to_char(100*Decode( p_distribution, 'M', p_sleep_times(1),  
			 		 p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(1))), '999,999,999') ||
			 	to_char(100*Decode( p_distribution, 'M', p_sleep_times(3),  
					 p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(2))), '999,999,999') ||
			 	to_char(100*Decode( p_distribution, 'M', p_sleep_times(5), 
					 p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(3))), '999,999,999') ||
			 	to_char(100*Decode( p_distribution, 'M', p_sleep_times(7), 
					 p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(4))), '999,999,999') ||
			 	to_char(100*Decode( p_distribution, 'M', p_sleep_times(9), 
					 p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(5))), '999,999,999') ||
			 	to_char(100*Decode( p_distribution, 'M', p_sleep_times(11), 
					 p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(6))), '999,999,999') ||
			 	to_char(100*Decode( p_distribution, 'M', p_sleep_times(13), 
					 p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(7))), '999,999,999') ||
			 	to_char(100*Decode( p_distribution, 'M', p_sleep_times(15), 
					 p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(8))), '999,999,999') ||
			 	to_char(100*Decode( p_distribution, 'M', p_sleep_times(17), 
					 p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(9))), '999,999,999') ||
			 	to_char(100*Decode( p_distribution, 'M', p_sleep_times(19), 
					 p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(10))), '999,999,999')
			into t_elapsed_time_ulast
			from dual;
			dbms_output.put_line( t_elapsed_time_ulast);

			-- Round(p_rampup_users(i),0) is better suited than Trunc(), but shell deals better with integers
			-- and so it's declared as typeset -i. Hence Trunc() is used here in pl/sql to keep it consistent
			SELECT	Rpad( '.d. Elapsed Time: LUser [cs] c', 30) ||
				To_Char( Sum( Decode( run_id, 1, ela 
					- 100*Decode( p_distribution, 'M', p_sleep_times(1),  
		       		 		      p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(1))),
					      		      0)), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 2, ela 
					- 100*Decode( p_distribution, 'M', p_sleep_times(3),  
		       		 		      p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(2))),
					      		      0)), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 3, ela 
					- 100*Decode( p_distribution, 'M', p_sleep_times(5),  
		       		 		      p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(3))),
					      		      0)), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 4, ela 
					- 100*Decode( p_distribution, 'M', p_sleep_times(7),  
		       		 		      p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(4))),
					      		      0)), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 5, ela 
					- 100*Decode( p_distribution, 'M', p_sleep_times(9),  
		       		 		      p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(5))),
					      		      0)), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 6, ela 
					- 100*Decode( p_distribution, 'M', p_sleep_times(11),  
		       		 		      p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(6))),
					      		      0)), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 7, ela 
					- 100*Decode( p_distribution, 'M', p_sleep_times(13),  
		       		 		      p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(7))),
					      		      0)), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 8, ela 
					- 100*Decode( p_distribution, 'M', p_sleep_times(15),  
		       		 		      p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(8))),
					      		      0)), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 9, ela 
					- 100*Decode( p_distribution, 'M', p_sleep_times(17),  
		       		 		      p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(9))),
					      		      0)), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 10, ela 
					- 100*Decode( p_distribution, 'M', p_sleep_times(19),  
		       		 		      p_rampup_users(11)-Trunc(p_rampup_users(11)/p_rampup_users(10))),
					      		      0)), '999,999,999') 
			INTO	t_elapsed_time_ulast
			FROM	(
				SELECT *
				FROM	(
					SELECT   
						 rm1.run_id
					--      ,rm1.iteration_id
					        ,rm1.user_id
					--	,rm1.create_datetime
			       		 	,rm1.concurrent_users
					--
				      		,((To_Char( rm1.create_datetime, 'SSSSS') - 
						   To_Char( rm2.create_datetime, 'SSSSS')
						  ) + 
						  (To_Char( rm1.create_datetime, 'FF2') - 
						   To_Char( rm2.create_datetime, 'FF2')
						  )/100
						 )*100 ela
					FROM    ( 
						SELECT * 
						FROM 	(
							SELECT   rm1.run_id
		  					--      ,rm1.iteration_id
							        ,rm1.user_id
							        ,rm1.create_datetime
						    	        ,rm1.concurrent_users
						        --
						                ,Max( rm1.create_datetime) OVER( PARTITION BY rm1.run_id ORDER BY rm1.run_id) max_datetime
						  	FROM     ssstest_user.ssstest_runstats_messages rm1 
							WHERE   rm1.user_id != 0
							)
						WHERE	create_datetime = max_datetime
						) rm1 					/* current */
					        ,ssstest_user.ssstest_runstats_messages rm2 /* previous */
					WHERE   rm2.user_id = 0
			       		    AND rm2.run_id = rm1.run_id-1
					ORDER BY rm1.run_id ASC
					)
				);
			dbms_output.put_line( t_elapsed_time_ulast);
		   END IF;

		   SELECT
			Rpad( '.d. Elapsed Time: FUser [cs]', 30) ||
			To_Char( Sum( Decode( run_id, 1, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id, 2, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id, 3, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id, 4, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id, 5, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id, 6, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id, 7, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id, 8, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id, 9, ela, 0)),		'999,999,999') ||
			To_Char( Sum( Decode( run_id,10, ela, 0)),		'999,999,999')
		   INTO	t_elapsed_time_flast
		   FROM	(
			SELECT *
			FROM	(
				SELECT   
					 rm1.run_id
				--      ,rm1.iteration_id
				        ,rm1.user_id
				--	,rm1.create_datetime
		       		 	,rm1.concurrent_users
				--
			      		,((To_Char( rm1.create_datetime, 'SSSSS') - 
					   To_Char( rm2.create_datetime, 'SSSSS')
					  ) + 
					  (To_Char( rm1.create_datetime, 'FF2') - 
					   To_Char( rm2.create_datetime, 'FF2')
					  )/100
					 )*100 ela
				FROM    ( 
					SELECT * 
					FROM 	(
						SELECT   rm1.run_id
	  					--      ,rm1.iteration_id
						        ,rm1.user_id
						        ,rm1.create_datetime
					    	        ,rm1.concurrent_users
					        --
					                ,Min( rm1.create_datetime) OVER( PARTITION BY rm1.run_id ORDER BY rm1.run_id) max_datetime
					  	FROM     ssstest_user.ssstest_runstats_messages rm1 
						WHERE   rm1.user_id != 0
						)
					WHERE	create_datetime = max_datetime
					) rm1 					/* current */
				        ,ssstest_user.ssstest_runstats_messages rm2 /* previous */
				WHERE   rm2.user_id = 0
		       		    AND rm2.run_id = rm1.run_id-1
				ORDER BY rm1.run_id ASC
				)
		   );
		   dbms_output.put_line( t_elapsed_time_flast);

		   IF ( p_distribution = 'M') THEN
			select  rpad('.d. sleep time (distr='||p_distribution||')......', 30) ||
			 	To_Char(100*p_sleep_times( 1), '999,999,999') ||
			 	To_Char(100*p_sleep_times( 2), '999,999,999') ||
			 	To_Char(100*p_sleep_times( 4), '999,999,999') ||
			 	To_Char(100*p_sleep_times( 6), '999,999,999') ||
			 	To_Char(100*p_sleep_times( 8), '999,999,999') ||
			 	To_Char(100*p_sleep_times(10), '999,999,999') ||
			 	To_Char(100*p_sleep_times(12), '999,999,999') ||
			 	To_Char(100*p_sleep_times(14), '999,999,999') ||
			 	To_Char(100*p_sleep_times(16), '999,999,999') ||
			 	To_Char(100*p_sleep_times(18), '999,999,999')
			into t_elapsed_time_flast
			from dual;
			dbms_output.put_line( t_elapsed_time_flast);

			SELECT	Rpad( '.d. Elapsed Time: FUser [cs] c', 30) ||
				To_Char( Sum( Decode( run_id, 1, ela, 0)) - 100*p_sleep_times( 1), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 2, ela, 0)) - 100*p_sleep_times( 2), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 3, ela, 0)) - 100*p_sleep_times( 4), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 4, ela, 0)) - 100*p_sleep_times( 6), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 5, ela, 0)) - 100*p_sleep_times( 8), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 6, ela, 0)) - 100*p_sleep_times(10), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 7, ela, 0)) - 100*p_sleep_times(12), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 8, ela, 0)) - 100*p_sleep_times(14), '999,999,999') ||
				To_Char( Sum( Decode( run_id, 9, ela, 0)) - 100*p_sleep_times(16), '999,999,999') ||
				To_Char( Sum( Decode( run_id,10, ela, 0)) - 100*p_sleep_times(18), '999,999,999')
			INTO	t_elapsed_time_flast
			FROM	(
				SELECT *
				FROM	(
					SELECT   
						 rm1.run_id
					--      ,rm1.iteration_id
				      		,rm1.user_id
					--	,rm1.create_datetime
			       		 	,rm1.concurrent_users
					--
			      			,((To_Char( rm1.create_datetime, 'SSSSS') - 
						   To_Char( rm2.create_datetime, 'SSSSS')
						  ) + 
						  (To_Char( rm1.create_datetime, 'FF2') - 
						   To_Char( rm2.create_datetime, 'FF2')
						  )/100
						 )*100 ela
					FROM    ( 
						SELECT * 
						FROM 	(
							SELECT   rm1.run_id
		  					--      ,rm1.iteration_id
							        ,rm1.user_id
							        ,rm1.create_datetime
						    	        ,rm1.concurrent_users
						        --
						                ,Min( rm1.create_datetime) OVER( PARTITION BY rm1.run_id ORDER BY rm1.run_id) max_datetime
						  	FROM     ssstest_user.ssstest_runstats_messages rm1 
							WHERE   rm1.user_id != 0
							)
						WHERE	create_datetime = max_datetime
						) rm1 					/* current */
					        ,ssstest_user.ssstest_runstats_messages rm2 /* previous */
					WHERE   rm2.user_id = 0
			       		    AND rm2.run_id = rm1.run_id-1
					ORDER BY rm1.run_id ASC
					)
			);
			dbms_output.put_line( t_elapsed_time_flast);
		   END IF;
		END IF; 	-- end of print 'alternative ELA response time' decision

	     	IF( p_report != 'RES') THEN	
                        SELECT  Rpad( '... Elapsed Time: BUser [cs]', 30) ||
                                To_Char( Sum( Decode( run_id, 1, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id, 2, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id, 3, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id, 4, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id, 5, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id, 6, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id, 7, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id, 8, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id, 9, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id,10, ela, 0)), '999,999,999')
                        INTO    t_elapsed_time_flast
			FROM	(
                                SELECT
                                         run_id
                                        ,Min( ela) ela          --Decode( ela, 0, NULL, ela)) ela
                                FROM    (
                                        SELECT
                                                 ( CASE WHEN run_id-100 < 0 THEN run_id ELSE run_id-100 END 
                                                 ) run_id
                                                ,( (To_Char( end_time, 'SSSSS') + To_Char( end_time, 'FF2')/100)
                                                 -
                                                      (To_Char( create_datetime, 'SSSSS') + To_Char( create_datetime, 'FF2')/100)
                                                )*100 ela
                                        FROM    (
                                                SELECT
                                                         run_id
                                                        ,user_id
                                                        ,create_datetime
                                                        ,(
                                                               Max(create_datetime) OVER (PARTITION BY (
                                                                        CASE WHEN run_id-100 < 0
                                                                             THEN run_id ELSE run_id-100 END ), user_id
                                                               )
                                                        ) end_time
                                                FROM    ssstest_user.ssstest_runstats_messages
                                                )
                                        )
                                        WHERE ela != 0
                                GROUP BY run_id
			  )
			;
                        dbms_output.put_line( t_elapsed_time_flast);

                       SELECT  Rpad( '... Elapsed Time: WUser [cs]', 30) ||
                                To_Char( Sum( Decode( run_id, 1, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id, 2, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id, 3, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id, 4, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id, 5, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id, 6, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id, 7, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id, 8, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id, 9, ela, 0)), '999,999,999') ||
                                To_Char( Sum( Decode( run_id,10, ela, 0)), '999,999,999')
                        INTO    t_elapsed_time_ulast
                        FROM    (
                                        SELECT
                                                 (CASE WHEN run_id-100 < 0 THEN run_id ELSE run_id-100 END
                                                 ) run_id
                                                ,Max( (To_Char( end_time, 'SSSSS') + To_Char( end_time, 'FF2')/100)
                                                 -
                                                      (To_Char( create_datetime, 'SSSSS') + To_Char( create_datetime, 'FF2')/100)
                                                )*100 ela
                                        FROM    (
                                                SELECT
                                                         run_id
                                                        ,user_id
                                                        ,create_datetime
                                                        ,(
                                                              Max(create_datetime) OVER (PARTITION BY (
                                                                        CASE WHEN run_id-100 < 0
                                                                             THEN run_id ELSE run_id-100 END ), user_id
                                                              )
                                                        ) end_time
                                                FROM    ssstest_user.ssstest_runstats_messages
                                                )
                                        GROUP BY
                                                (
                                                      CASE WHEN run_id-100 < 0 THEN run_id ELSE run_id-100 END
                                                )
                                );
                        dbms_output.put_line( t_elapsed_time_ulast);
		END IF; 	-- end of print 'primary ELA response time' decision

	     	IF( p_report = 'DBG') THEN	
		   	WITH ela_by_user AS (
                                        SELECT
                                                 ( CASE WHEN run_id-100 < 0 THEN run_id ELSE run_id-100 END
                                                 ) run_id
						,user_id
                                                ,( (To_Char( end_time, 'SSSSS') + To_Char( end_time, 'FF2')/100)
                                                 -
                                                      (To_Char( create_datetime, 'SSSSS') + To_Char( create_datetime, 'FF2')/100)
                                                )*100 ela
                                        FROM    (
                                                SELECT
                                                         run_id
                                                        ,user_id
                                                        ,create_datetime
                                                        ,(
                                                               Max(create_datetime) OVER (PARTITION BY (
                                                                        CASE WHEN run_id-100 < 0
                                                                             THEN run_id ELSE run_id-100 END ), user_id
                                                               )
                                                        ) end_time
                                                FROM    ssstest_user.ssstest_runstats_messages
                                        )
		   	)
                        SELECT 	Rpad( '... Elapsed Time: FUser [cs] c', 30) ||
                                To_Char( Sum( Decode( run_id, 1, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 1, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id, 2, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 2, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id, 3, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 3, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id, 4, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 4, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id, 5, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 5, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id, 6, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 6, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id, 7, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 7, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id, 8, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 8, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id, 9, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 9, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id,10, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id,10, user_id, 0)), '09')) ||')'
                        INTO    t_elapsed_time_flast
			FROM	(
				   SELECT *
				   FROM	 (
				   SELECT
					 run_id
					,user_id
					,ela
					,Min(ela) OVER (PARTITION BY run_id) min_ela
				   FROM	ela_by_user
                        	   WHERE
					ela != 0
				   )
				   WHERE	ela = min_ela
			);
                        dbms_output.put_line( t_elapsed_time_flast);

		   	WITH ela_by_user AS (
                                           SELECT
                                                 (CASE WHEN run_id-100 < 0 THEN run_id ELSE run_id-100 END
                                                 ) run_id
						,user_id
                                                ,( (To_Char( end_time, 'SSSSS') + To_Char( end_time, 'FF2')/100)
                                                 -
                                                      (To_Char( create_datetime, 'SSSSS') + To_Char( create_datetime, 'FF2')/100)
                                                )*100 ela
                                           FROM    (
                                                SELECT
                                                         run_id
                                                        ,user_id
                                                        ,create_datetime
                                                        ,(
                                                              Max(create_datetime) OVER (PARTITION BY (
                                                                        CASE WHEN run_id-100 < 0
                                                                             THEN run_id ELSE run_id-100 END ), user_id
                                                              )
                                                        ) end_time
                                                FROM    ssstest_user.ssstest_runstats_messages
                                                )
                        )
                        SELECT  Rpad( '... Elapsed Time: LUser [cs] c', 30) ||
                                To_Char( Sum( Decode( run_id, 1, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 1, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id, 2, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 2, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id, 3, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 3, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id, 4, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 4, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id, 5, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 5, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id, 6, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 6, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id, 7, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 7, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id, 8, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 8, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id, 9, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id, 9, user_id, 0)), '09')) ||')'||
                                To_Char( Sum( Decode( run_id,10, ela, 0)), '999,999') ||'('||
                                Ltrim( To_Char( Sum( Decode( run_id,10, user_id, 0)), '09')) ||')'
                        INTO    t_elapsed_time_ulast
                        FROM    (
                                   SELECT *
                                   FROM  (
                                   SELECT
                                         run_id
                                        ,user_id
                                        ,ela
                                        ,Max(ela) OVER (PARTITION BY run_id) max_ela
                                   FROM ela_by_user
                                   WHERE
                                        ela != 0
                                   )
                                   WHERE        ela = max_ela
			);
                        dbms_output.put_line( t_elapsed_time_ulast);
		END IF; 	-- end of print 'primary debug ELA response time' decision

		prc_draw_line;

		dbms_output.put_line
		    ( rpad( 'Resources (stat/latch) & Ela', 30 ) || -- lpad( 'Run1', 12 ) || 
		      -- lpad( 'Run2', 12 ) || lpad( 'Run3', 12 ) || lpad( 'Run4', 12 ) || lpad( 'Run5', 12 ) || 
		      lpad( 'Run1', 6 ) || ' - ' || lpad( p_rampup_users(1), 3, '0') || 
		      lpad( 'Run2', 6 ) || ' - ' || lpad( p_rampup_users(2), 3, '0') || 
		      lpad( 'Run3', 6 ) || ' - ' || lpad( p_rampup_users(3), 3, '0') ||
		      lpad( 'Run4', 6 ) || ' - ' || lpad( p_rampup_users(4), 3, '0') ||
		      lpad( 'Run5', 6 ) || ' - ' || lpad( p_rampup_users(5), 3, '0') ||
		      lpad( 'Run6', 6 ) || ' - ' || lpad( p_rampup_users(6), 3, '0') ||
		      lpad( 'Run7', 6 ) || ' - ' || lpad( p_rampup_users(7), 3, '0') ||
		      lpad( 'Run8', 6 ) || ' - ' || lpad( p_rampup_users(8), 3, '0') ||
		      lpad( 'Run9', 6 ) || ' - ' || lpad( p_rampup_users(9), 3, '0') ||
		      lpad( 'Run10', 6 ) || ' - ' || lpad( p_rampup_users(10), 3, '0')  );
	ELSE
		dbms_output.put_line('*** Error: Only 2, 3, 4, 5 or 10 runs are allowed. Received number of runs=' || t_runs);
	END IF;

  END prc_results;


  PROCEDURE prc_service_time_snap (
	 p_run_id	 INTEGER
	,p_flg_begin_end INTEGER
  ) AS
	t_timing	ssstest_user.ssstest_runstats_service_time.timing%TYPE;
  BEGIN
	IF (p_run_id = 1 AND p_flg_begin_end = 1) THEN
		DELETE ssstest_user.ssstest_runstats_service_time;
		COMMIT;
	END IF;

	t_timing := dbms_utility.get_time;
	INSERT INTO ssstest_user.ssstest_runstats_service_time ( run_id, flg_begin_end, timing)
	VALUES( p_run_id, p_flg_begin_end, t_timing);
	COMMIT;
  END prc_service_time_snap;


  PROCEDURE prc_service_time_results (
	 p_runs		INTEGER
  ) AS
	t_time_begin	INT;	--NUMBER(10,3);
	t_time_end	INT;
	-- t_time_per_run	INT; 
	t_time_total	INT 	:= 0;
	t_std		INT	:= 0;

	TYPE time_per_run_ty IS TABLE OF INT;
	times_per_run	time_per_run_ty := time_per_run_ty();
	t_time_string	VARCHAR2(1000) := '';
  BEGIN
	FOR i IN 1..p_runs LOOP
		EXECUTE IMMEDIATE 'SELECT timing FROM ssstest_user.ssstest_runstats_service_time WHERE run_id=:1 and flg_begin_end=2'
		INTO t_time_end USING i;

		EXECUTE IMMEDIATE 'SELECT timing FROM ssstest_user.ssstest_runstats_service_time WHERE run_id=:1 and flg_begin_end=1'
		INTO t_time_begin USING i;

		-- t_time_per_run := t_time_end - t_time_begin;
		times_per_run.EXTEND;
		times_per_run(i) := t_time_end - t_time_begin;
		-- dbms_output.put_line( 'Run#'|| i || ' - ' || t_time_per_run);
		-- t_time_total := t_time_total + t_time_per_run;
		t_time_total := t_time_total + times_per_run(i);
		t_time_string := t_time_string || ' ' || times_per_run(i);
	END LOOP;
	t_time_total := t_time_total/p_runs;
	FOR i IN 1..times_per_run.LAST LOOP	-- or just p_runs
		t_std := t_std + power( t_time_total - times_per_run(i), 2);
	END LOOP;
	t_std := sqrt(t_std/p_runs);
	dbms_output.put_line( '*** Service Time measurements = (' || t_time_string || ' )' );
	dbms_output.put_line( '*** First Run = ' || times_per_run(1) || ' [csec]');
	dbms_output.put_line( '*** Arithmetic mean and standard deviation over ' || p_runs || ' runs');
	dbms_output.put_line( '*** M +/- STD (service time) = ' || t_time_total || ' +/- ' || t_std || ' [csec]');
  END;


  PROCEDURE prc_draw_line 
  AS
  BEGIN
		   dbms_output.put_line( 
			rpad('-', 30, '-') || ' ' ||
			rpad('-', 11, '-') || ' ' ||
			rpad('-', 11, '-') || ' ' ||
			rpad('-', 11, '-') || ' ' ||
			rpad('-', 11, '-') || ' ' ||
			rpad('-', 11, '-') || ' ' || -- end of 5 runs
			rpad('-', 11, '-') || ' ' ||
			rpad('-', 11, '-') || ' ' ||
			rpad('-', 11, '-') || ' ' ||
			rpad('-', 11, '-') || ' ' ||
			rpad('-', 11, '-') );
  END prc_draw_line;

END pkg_ssstest_runstats;
/

sho err

CREATE or replace PUBLIC SYNONYM pkg_ssstest_runstats FOR ssstest_user.pkg_ssstest_runstats;
GRANT EXECUTE ON pkg_ssstest_runstats TO scott;

set def on


/********************* Sample of use:
set lin 150 serveroutput on size 100000
exec pkg_runstats.prc_snap( 0, true)
select d.deptno, e.ename from scott.dept d, scott.emp e where d.deptno=e.deptno;
exec pkg_runstats.prc_snap( 1)
select d.deptno, e.ename from scott.dept d, scott.emp e where d.deptno=e.deptno;
exec pkg_runstats.prc_snap( 2)
select e.deptno, e.ename from scott.dept d, scott.emp e where d.deptno=e.deptno;
exec pkg_runstats.prc_snap( 3)
select e.deptno, e.ename from scott.dept d, scott.emp e where d.deptno=e.deptno;
exec pkg_runstats.prc_snap( 4)
select e.deptno, e.ename from scott.dept d, scott.emp e where d.deptno=e.deptno;
exec pkg_runstats.prc_snap( 5)
exec pkg_runstats.prc_results
**************************************/

