
-- DB objects (tables and indexes) specific to the test-case:
-- drop table ssstest_user.emp_big;
-- drop table ssstest_user.dept_big;

CREATE TABLE ssstest_user.dept_big(
	deptno		NOT NULL,
	dname,
	loc_mod,
	loc_trunc
)
NOLOGGING
AS
WITH rows_1k AS (
	SELECT --+ materialize
		rownum
	FROM	all_objects
	WHERE	rownum <= 1000
)
SELECT
	 rownum-1
	,Rpad( rownum-1, 10)
--	,Rpad( rownum, 10)
	,Mod( rownum-1, 10)	/*     10 primary locations */
	,Trunc((rownum-1)/1000)	/*     10 alternative locations */
FROM	 rows_1k
	,rows_1k
WHERE 	rownum <= 10000		/*  10,000 departments */
/

ALTER TABLE ssstest_user.dept_big ADD CONSTRAINT dept_big_pk PRIMARY KEY (deptno);
CREATE INDEX ssstest_user.dept_big_loc_mod_idx ON ssstest_user.dept_big(loc_mod);
CREATE INDEX ssstest_user.dept_big_loc_trunc_idx ON ssstest_user.dept_big(loc_trunc);

CREATE TABLE ssstest_user.emp_big(
	empno		NOT NULL,
	ename,
	job,
	mgr,
	hiredate,
	sal,
	comm,
	deptno_mod,
	deptno_trunc
)
NOLOGGING
AS
WITH rows_10k AS (
	SELECT --+ materialize
		rownum
	FROM	all_objects
	WHERE	rownum <= 10000
)
SELECT
	 rownum			/* 1,000,000 */
	,Rpad( rownum, 7)	/* 1,000,000 */
	,Rpad( rownum, 7)	/* 1,000,000 */
	,Mod( rownum, 10000)	/*    10,000 */
	,sysdate
	,Trunc((rownum-1)/1000)	/*     1,000 */
	,Trunc((rownum-1)/100)	/*    10,000 */
	,Mod( rownum, 10000)	/*    10,000 */
	,Trunc((rownum-1)/100)	/*    10,000 */
FROM	 rows_10k
	,rows_10k
WHERE	rownum <= 1000000	/* 1,000,000 emp -> 1,000,000 emp/10,000 dept = 100 employees per dept */
/

ALTER TABLE ssstest_user.emp_big ADD CONSTRAINT emp_big_pk PRIMARY KEY (empno);
ALTER TABLE ssstest_user.emp_big ADD CONSTRAINT emp_big_mod_fk FOREIGN KEY (deptno_mod) REFERENCES ssstest_user.dept_big (deptno);
CREATE INDEX ssstest_user.emp_big_deptno_mod_idx ON ssstest_user.emp_big(deptno_mod);

ALTER TABLE ssstest_user.emp_big ADD CONSTRAINT emp_big_trunc_fk FOREIGN KEY (deptno_trunc) REFERENCES ssstest_user.dept_big (deptno);
CREATE INDEX ssstest_user.emp_big_deptno_trunc_idx ON ssstest_user.emp_big(deptno_trunc);

exec dbms_stats.gather_table_stats( 'ssstest_user', 'dept_big', cascade=>true)
exec dbms_stats.gather_table_stats( 'ssstest_user', 'emp_big', cascade=>true)

CREATE OR REPLACE VIEW ssstest_user.v_dept_emp_big_mod AS
SELECT  d.*, e.*
FROM    ssstest_user.dept_big d, ssstest_user.emp_big e
WHERE   d.deptno=e.deptno_mod
/
CREATE OR REPLACE VIEW ssstest_user.v_dept_emp_big_trunc AS
SELECT  d.*, e.*
FROM    ssstest_user.dept_big d, ssstest_user.emp_big e
WHERE   d.deptno=e.deptno_trunc
/
CREATE OR REPLACE VIEW ssstest_user.v_dept_emp_big_trunc_hint AS
SELECT  /*+ index(e emp_big_deptno_trunc_idx) */ d.*, e.*
FROM    ssstest_user.dept_big d, ssstest_user.emp_big e
WHERE   d.deptno=e.deptno_trunc
/

GRANT SELECT ON ssstest_user.v_dept_emp_big_mod TO scott;
GRANT SELECT ON ssstest_user.v_dept_emp_big_trunc TO scott;
GRANT SELECT ON ssstest_user.v_dept_emp_big_trunc_hint TO scott;
CREATE or replace PUBLIC SYNONYM v_dept_emp_big_mod FOR ssstest_user.v_dept_emp_big_mod;
CREATE or replace PUBLIC SYNONYM v_dept_emp_big_trunc FOR ssstest_user.v_dept_emp_big_trunc;
CREATE or replace PUBLIC SYNONYM v_dept_emp_big_trunc_hint FOR ssstest_user.v_dept_emp_big_trunc_hint;


set autotrace on explain timi on
prompt *** Original HJ: select count(*), avg(sal) from v_dept_emp_big_trunc where loc_trunc between 4 and 5;
select count(*), avg(sal) from v_dept_emp_big_trunc where loc_trunc between 4 and 5;
prompt *** Hinted/"Tuned" NL: select count(*), avg(sal) from v_dept_emp_big_trunc_hint where loc_trunc between 4 and 5;
select count(*), avg(sal) from v_dept_emp_big_trunc_hint where loc_trunc between 4 and 5;
set autotrace off timi off


set doc off
DOC
/************
-- Test single-user and multi-user ramp-up of 1,5,10,15,20 to check if indeed
-- access path (FTS or FFIS) does NOT affect AT ALL scalability with respect to concurrent user-count:
select count(1) from ssstest_user.emp_big e;
select /*+ full(e) */ count(2) from ssstest_user.emp_big e;

grant select on ssstest_user.emp_big to scott;
create public synonym emp_big for ssstest_user.emp_big;


-- Query:
-- as ssstest_user_user0:
alter session set optimizer_index_caching=0;
alter session set optimizer_index_cost_adj=100;
-- alter session set tracefile_identifier='wrong_card_est_user0';
-- alter session set events '10046 trace name context forever, level 12:10132 trace name context forever';
-- alter session set events '10053 trace name context forever';

select count(deptno || dname || empno || ename || job) from v_dept_emp_big_mod where deptno between 10 and 11;

select count(deptno || dname || empno || ename || job) from v_dept_emp_big_mod where deptno_mod = 10;
select count(deptno || dname || empno || ename || job) from v_dept_emp_big_mod where deptno_mod between 10 and 11;
select count(deptno_mod || dname || empno || ename || job) from v_dept_emp_big_mod where deptno_mod between 10 and 11;

select count(1) from v_dept_emp_big_mod where deptno_trunc=1;

select count(1) from v_dept_emp_big_trunc where deptno=1;
select count(1) from v_dept_emp_big_trunc where deptno_mod=1;
select count(1) from v_dept_emp_big_trunc where deptno_trunc=1;

select * from dual /* just to ensure cursor is closed */
/


-- as ssstest_user_user2:
alter session set optimizer_index_caching=0;
alter session set optimizer_index_cost_adj=100;
alter session set tracefile_identifier='wrong_card_est_user2';
alter session set events '10046 trace name context forever, level 12:10132 trace name context forever';
alter session set events '10053 trace name context forever';

alter session set optimizer_dynamic_sampling = 1;

SELECT	/* ssstest_user_user2: dynamic_sampling=3 */ count(22)
FROM	ssstest_user.dept_big d, ssstest_user.emp_big e
WHERE	d.deptno=e.deptno 
/

alter session set optimizer_dynamic_sampling = 3;

SELECT	/* ssstest_user_user2: dynamic_sampling=3 */ count(22)
FROM	ssstest_user.dept_big d, ssstest_user.emp_big e
WHERE	d.deptno=e.deptno 
/

select * from dual /* just to ensure cursor is closed */
/

-- How many employees in each department in classic SCOTT's schema?

ssstest_user@dani10> select d.deptno, d.dname, count(e.ename) from scott.dept d, scott.emp e where d.deptno = e.deptno(+) group by d.deptno, d.dname;

    DEPTNO DNAME          COUNT(E.ENAME)
---------- -------------- --------------
        10 ACCOUNTING                  3
        20 RESEARCH                    5
        30 SALES                       6
        40 OPERATIONS                  0

4 rows selected.
***********************/
#







