#!/usr/bin/ksh

# SSS-test auxiliary script
# Dali Database Solutions Inc. (C) Copyright. All rights reserved.
# http://www.tns-0.com

# This script is designed for no more than N runs, where 1 <= N <= 5
# In fact it executes 2 tests: fist under stateLESS paradigm, than stateFULL
# This script is still controled by ssstest.par (just like the main sss-test), yet the state flag is ignored
# N.B: This test works for Zero (login-all-at-once) and Deterministic distributions, but not for M (exponential) distribution

###############################################
## Function fnc_stateless_run_test
###############################################
function fnc_stateless_run_test {
	print -n "* Start Run#=$T_RUN_ID, User#=$T_USER_ID ..."
	$ORACLE_HOME/bin/sqlplus -s /nolog <<EOF
	set term off feed off
	connect $ORACLE_USERNAME/$ORACLE_PASSWORD
	@$QUERY_FILE
	INSERT INTO ssstest_user.ssstest_runstats_messages( run_id, iteration_id, user_id, concurrent_users)
	VALUES( $T_RUN_ID, $T_ITERATION_ID, $T_USER_ID, $P_CONCURRENT_USERS );
	commit;
	exit
EOF
}

###############################################
## Function fnc_COPput
###############################################
function fnc_COPput { print -p -- "$@"; }

###############################################
## Function fnc_COPget
###############################################
function fnc_COPget {
        while read -p -r LINE ;do
                [[ "$LINE" == "endInput" ]] && break
                print -- $LINE
        done
        # debug: ps | grep sqlplus
}

###############################################
## Function fnc_start
###############################################
function fnc_start {
        $ORACLE_HOME/bin/sqlplus -s /nolog <<EOF
        connect $ORACLE_USERNAME/$ORACLE_PASSWORD
        set serveroutput on feed off
        delete ssstest_user.ssstest_runstats_messages;
        commit;
        exit 3
EOF
#       [ $? == 3 ] || { print "\nProblem with connectivity to the  DB $ORACLE_SID"; exit 1; }

	# fnc_COPstart:
	# print "* debug: Launch co-process (COP) to:" 
	# print "* 1) trigger/release a run 2) record start of login/query (in stateless) or just query (in statefull)."
	# Accepts as many messages per run as number of users in it:
	$ORACLE_HOME/bin/sqlplus -s $ORACLE_USERNAME/$ORACLE_PASSWORD |&       COPpid=$!
	# print "debug: COPpid=$COPpid"
}

###############################################
## Function fnc_report
###############################################
function fnc_report {
        $ORACLE_HOME/bin/sqlplus -s /nolog <<EOF
        connect $ORACLE_USERNAME/$ORACLE_PASSWORD
        set lin 150 serveroutput on size 100000
        declare
                -- t_rampup_users       ssstest_user.ot_varray_rampup_users := ssstest_user.ot_varray_rampup_users();
                t_rampup_users  ssstest_user.ot_varray_rampup_users;
                t_sleep_times   ssstest_user.ot_varray_sleep_times;
        begin
                -- By convention, the last element of the collection is the duration:
                IF( $P_RUNS = 5) THEN
			-- content of runs[@] is identical for the first 5 runs (statless) and the last 5 runs (statefull):
			t_rampup_users := ssstest_user.ot_varray_rampup_users( ${runs[0]}, ${runs[1]}, ${runs[2]}, ${runs[3]}, ${runs[4]}, ${runs[0]}, ${runs[1]}, ${runs[2]}, ${runs[3]}, ${runs[4]}, $P_DURATION);
                ELSE
                        dbms_output.put_line('*** Error in fnc_report ' || $P_RUNS);
                END IF;
                ssstest_user.pkg_ssstest_runstats_state.prc_results( '$P_DISTRIBUTION', t_rampup_users);
	end;
/
        exit
EOF
}

###############################################
## Function fnc_process_input_para
###############################################
function fnc_process_input_para {
	grep "^[^#]" $PARA_FILE > $TMP_PARA_FILE
	# below typeset is probably not needed (used typeset, as declare is bash specific):
	typeset -i paraLINE_NUMBER=0
	print "Test Parameters Summary ($(date)):"
	# Read 3 uncommented parameter lines (see .par file for detailed structure):
	# sed '/^#/d' $PARA_FILE | 
	while read FULL_LINE ;do
		((paraLINE_NUMBER+=1))
		# print "Line# $paraLINE_NUMBER: $FULL_LINE"
		# LINE_${paraLINE_NUMBER}=$FULL_LINE
		if [ paraLINE_NUMBER -eq 1 ] ;then
			ORACLE_SID=`print $FULL_LINE | cut -f1 -d':'`
			ORACLE_HOME=`print $FULL_LINE | cut -f2 -d':'`
			ORACLE_USERNAME=`print $FULL_LINE | cut -f3 -d':'`
			ORACLE_PASSWORD=`print $FULL_LINE | cut -f4 -d':'`
			print "$paraLINE_NUMBER: OSID=$ORACLE_SID, OHome=$ORACLE_HOME, OUsername/OPassword=$ORACLE_USERNAME/$ORACLE_PASSWORD"
			export ORACLE_SID ORACLE_HOME ORACLE_USERNAME ORACLE_PASSWORD
		elif [ paraLINE_NUMBER -eq 2 ] ;then
			P_STATE=$(print $FULL_LINE | cut -f1 -d ':')
			P_FREQUENCY_PER_DURATION=$(print $FULL_LINE | cut -f2 -d ':')
			P_DURATION=$(print $FULL_LINE | cut -f3 -d ':')
			P_ITERATIONS=$(print $FULL_LINE | cut -f4 -d ':')
			P_DISTRIBUTION=$(print $FULL_LINE | cut -f5 -d ':')
			P_LET_RUN_AWAY=$(print $FULL_LINE | cut -f6 -d ':')
			P_EXPECTED_SERVICE_TIME=$(print $FULL_LINE | cut -f7 -d ':')
                        P_REPORT=$(print $FULL_LINE | cut -f8 -d ':')
			print "$paraLINE_NUMBER: STATE=$P_STATE (but doesn't matter for this test), FPD=$P_FREQUENCY_PER_DURATION, DUR=$P_DURATION, ITER=$P_ITERATIONS, DISTR=$P_DISTRIBUTION, RUN_AWAY=$P_LET_RUN_AWAY, EST=$P_EXPECTED_SERVICE_TIME, REPORT=$P_REPORT"
		elif [ paraLINE_NUMBER -eq 3 ] ;then

			# do nothing, let next routine figure it out
			set -A runs $(tail -1 $TMP_PARA_FILE)
			# Total number of Runs:
			P_RUNS="${#runs[@]}"
			print "$paraLINE_NUMBER: Number of runs (detailed user ramp-up): $P_RUNS (${runs[@]})"
			[ P_RUNS -gt 5 ] && { print "Number of runs for the stateLESS vs. stateFULL comparision can't exceed 5"; fnc_exit; }
		elif [ paraLINE_NUMBER -eq 4 ] ;then
			print "Problem with the parameter file. Too many uncommented input lines"; exit 1;
		else
			print "Trouble with parsing parameter file - Line# $paraLINE_NUMBER: $FULL_LINE"
		fi
	done < $TMP_PARA_FILE
}

###############################################
## Function fnc_all_at_once_distribution (Zero No-distribution)
###############################################
function fnc_all_at_once_distribution {
	typeset -i T_RUND_ID_FAKE=0
			# FREQUENCY PER DURATION: 2nd internal loop:
			T_FREQUENCY_PER_DURATION_ID=1
			while [ T_FREQUENCY_PER_DURATION_ID -le P_FREQUENCY_PER_DURATION ] ;do
				((T_SLEEP_TIME=P_DURATION/P_FREQUENCY_PER_DURATION))
				# print "*** SleepTime=$T_SLEEP_TIME (calculated as Duration/FPD=$P_DURATION/$P_FREQUENCY_PER_DURATION)"
					print "*** RUN=$T_RUN_ID, ConcurrentUsers=$P_CONCURRENT_USERS, ITERATION=$T_ITERATION_ID, FPD=$T_FREQUENCY_PER_DURATION_ID"
					T_USER_ID=1
					while [ T_USER_ID -le P_CONCURRENT_USERS ]; do
						((T_RUN_ID_FAKE=T_RUN_ID+100))
						fnc_COPput "set feed off"
						fnc_COPput "INSERT INTO ssstest_user.ssstest_runstats_messages( run_id, iteration_id, user_id, concurrent_users)"
						fnc_COPput "VALUES( $T_RUN_ID_FAKE, $T_ITERATION_ID, $T_USER_ID, $P_CONCURRENT_USERS );"
						fnc_COPput "commit;"
              					fnc_COPput "prompt endInput"
						fnc_COPget

						if [ $P_STATE = 'F' ] ;then
							fnc_statefull_go &
						elif [ $P_STATE = 'L' ] ;then
							fnc_stateless_run_test &
						else
							print "Wrong StateFull/StateLess flag. Value supplied is $P_STATE"; fnc_exit;
						fi
						((T_USER_ID+=1))
					done
				((T_FREQUENCY_PER_DURATION_ID+=1))
			done
}

###############################################
## Function fnc_deterministic_distribution
###############################################
function fnc_deterministic_distribution {
	typeset -i T_PACING
	typeset -i T_RUND_ID_FAKE=0
			# FREQUENCY PER DURATION: 2nd internal loop:
			T_FREQUENCY_PER_DURATION_ID=1
			while [ T_FREQUENCY_PER_DURATION_ID -le P_FREQUENCY_PER_DURATION ] ;do
				((T_SLEEP_TIME=P_DURATION/P_FREQUENCY_PER_DURATION))
				# print "*** SleepTime=$T_SLEEP_TIME (calculated as Duration/FPD=$P_DURATION/$P_FREQUENCY_PER_DURATION)"
					print "*** RUN=$T_RUN_ID, ConcurrentUsers=$P_CONCURRENT_USERS, ITERATION=$T_ITERATION_ID, FPD=$T_FREQUENCY_PER_DURATION_ID"
					# Calculate user login time/pacing:
					((T_PACING=P_DURATION/P_CONCURRENT_USERS))
					print "* PACING=$T_PACING [sec]"
					T_USER_ID=1
					while [ T_USER_ID -le P_CONCURRENT_USERS ]; do
						# Start the clock/record each user's start time (T_RUN_ID_FAKE), regardless of stateLess or StateFull:
						# (later record each user's end time (T_RUN_ID) in fnc_statefull_go or fnc_stateless_run_test routine)
						((T_RUN_ID_FAKE=T_RUN_ID+100))
						# print "debug: t_rund_id_fake=$T_RUN_ID_FAKE"
						fnc_COPput "set feed off"
						fnc_COPput "INSERT INTO ssstest_user.ssstest_runstats_messages( run_id, iteration_id, user_id, concurrent_users)"
						fnc_COPput "VALUES( $T_RUN_ID_FAKE, $T_ITERATION_ID, $T_USER_ID, $P_CONCURRENT_USERS );"
						fnc_COPput "commit;"
              					fnc_COPput "prompt endInput"
						fnc_COPget

						if [ $P_STATE = 'F' ] ;then
							fnc_statefull_go &
						elif [ $P_STATE = 'L' ] ;then
							fnc_stateless_run_test &
						else
							print "Wrong StateFull/StateLess flag. Value supplied is $P_STATE"; fnc_exit;
						fi
						## ((T_USER_ID+=1)) ## Use these 2 lines below if sleep is governed in fnc_main, not here:
						## [[ T_USER_ID -le P_CONCURRENT_USERS ]] && { print "sleeping for $T_PACING [sec]"; sleep $T_PACING; }
						print "sleeping for $T_PACING sec"; sleep $T_PACING;
						((T_USER_ID+=1))
					done
				((T_FREQUENCY_PER_DURATION_ID+=1))
			done
}

###############################################
## Function fnc_main
###############################################
function fnc_main {
	typeset -i T_RUN_ID=1
	typeset -i T_USER_ID
	typeset -i T_FREE_TO_GO
	typeset -i T_ITERATION_ID
	typeset -i T_FREQUENCY_PER_DURATION_ID

	typeset -u T_GENERATE_EXPONDIST_FLG
	set -A glb_sleep_times		# Global array to remember sleep times of first and last user for all runs
	typeset -i T_GLB_SLEEP_TIME=1	# Global subscript for the array above
	typeset -i T_ALREADY_LOGIN_USERS
	typeset -i T_PS_STATELESS=10	# anything greather than 1

	########## Stuff below is not needed any longer as now it's done in fnc_process_input_para ()
	# Line below works in Bash, but not in KSH (pdksh), but it's needed they way it is, otherwise gives wrong results:
	# e.g. wo/ extra paranthesis you get a string, not an array of elements:
	### set -A runs $(tail -1 $PARA_FILE)
	# Total number of Runs:
	### P_RUNS="${#runs[@]}"

	fnc_start

	# cleanup stateFULL regardless of the flag in ssstest.par:
	fnc_statefull_cleanup

     # This is the difference with the main sss-test: execute this test twice - first as stateLESS, than statFULL:
     typeset -i T_TEST_ID=1
     while [ T_TEST_ID -le 2 ] ;do
	if [ $T_TEST_ID == 1 ] ;then
		print "***** Test 1: StateLESS *****"; P_STATE='L'; T_RUN_ID=1
	else
		# At this point all previous stateLESS background processes suppose to be gone:
		while [ T_PS_STATELESS -gt 1 ] ;do
			sleep 1; T_PS_STATELESS=$(ps | grep -c sqlplus)
		done
		# debug: ps -fjH
		print "***** Test 2: StateFULL *****"; P_STATE='F'; T_RUN_ID=1001
	fi
	# RUNS: the most external loop on number of users (last line in $PARA_FILE) to test - "ramp up":
	for element in $(seq 0 $((${#runs[@]} - 1))); do
		P_CONCURRENT_USERS=${runs[$element]}
		T_ITERATION_ID=1
		# ITERATIONS: 1st internal loop:
		while [ T_ITERATION_ID -le P_ITERATIONS ]; do
			# In this test, launch 'em regardless of the flag...:
			if [ $P_STATE = 'F' ] ;then
				# debug: ps -fjH
				# Launch $P_CONCURRENT_USERS sqlplus sessions each blocked on dequeueing...
				T_USER_ID=1
				while [ T_USER_ID -le P_CONCURRENT_USERS ]; do
					# Has to be a background process, because each one gets blocked, so unless background, it waits indefinitely:
					fnc_statefull_login &
					((T_USER_ID+=1))
				done

				## Check whether or not all users participating in a run logged in? Cool off and continue only when everybody is in
				## If yes, one way is to loop until ssstest_login.count(1)=$P_CONCURRENT_USERS
				T_ALREADY_LOGIN_USERS=0
				while [ T_ALREADY_LOGIN_USERS -ne P_CONCURRENT_USERS ] ;do
					# less than elegant, but works :-(
				 	fnc_COPput "set term off"
					fnc_COPput "spool $TMP_ALREADY_LOGIN_USERS_FILE"
					# fnc_COPput "SELECT Count(*) FROM ssstest_user.ssstest_login WHERE run_id=$T_RUN_ID;"
					fnc_COPput "@ssstest_aux.sql $T_RUN_ID"
					fnc_COPput "spool off"
              				fnc_COPput "prompt endInput"
					fnc_COPget
                                        T_ALREADY_LOGIN_USERS=$(tail -1 $TMP_ALREADY_LOGIN_USERS_FILE)
                                        if [ $P_REPORT = 'DBG' ] ;then
                                                # print "debug: already logged-in users for run=$T_RUN_ID is $T_ALREADY_LOGIN_USERS"
                                                print -n "+"
                                        fi
				done
			fi

			case $P_DISTRIBUTION in 
			D) 
				print "* Deterministic Distribution"
				fnc_deterministic_distribution	 	## & used only if sleep in managed here, not in fnc_deterministic_distr
				;;
			Z)
				print "* Zero (All-at-once) no-Distribution"
				fnc_all_at_once_distribution &
				sleep $P_DURATION
				;;
			*)
				print "***Error: Wrong Distribution. Correct the parameter file. Value received is $P_DISTRIBUTION"; fnc_exit;
			esac
			((T_ITERATION_ID+=1))
		done
		((T_RUN_ID+=1))
	done
     	((T_TEST_ID+=1))
     done
}

##############################################
## Function fnc_statefull_cleanup
###############################################
function fnc_statefull_cleanup {
	$ORACLE_HOME/bin/sqlplus -s /nolog <<EOF
	connect $ORACLE_USERNAME/$ORACLE_PASSWORD
	set feed off
	delete ssstest_user.ssstest_queue_table;
	delete ssstest_user.ssstest_login;
	commit;
	exit
EOF
}

##############################################
## Function fnc_statefull_go
## Enqueue message/payload to trigger execution by a blocked dequeueing session
###############################################
function fnc_statefull_go {
        fnc_COPput "set serveroutput on feed off"
        # exec 4> /tmp/1.sql
        fnc_COPput "declare
                        enqueue_options       dbms_aq.enqueue_options_t;
                        message_properties    dbms_aq.message_properties_t;
                        message_handle        RAW(16);
                        message               ssstest_user.ot_message;
                begin
                        message := ssstest_user.ot_message( $T_RUN_ID, $T_ITERATION_ID, $T_USER_ID, localtimestamp);
                        dbms_aq.enqueue(
                                queue_name            => 'ssstest_user.ssstest_queue',
                                enqueue_options       => enqueue_options,
                                message_properties    => message_properties,
                                payload               => message,
                                msgid                 => message_handle);
                        commit;
                end;" ##  >&4
        # exec 4<&-
        fnc_COPput "/"
        # debug: fnc_COPput "list"
        fnc_COPput "prompt endInput"
        fnc_COPget
}

##############################################
## Function fnc_statefull_login
###############################################
function fnc_statefull_login {
	$ORACLE_HOME/bin/sqlplus -s /nolog <<EOF
	connect $ORACLE_USERNAME/$ORACLE_PASSWORD
	set serveroutput on term off feed off
	var t_run_id 		number
	var t_iteration_id 	number
	var t_user_id 		number

	-- get blocked until Poisson says ready to go:
	-- debug: begin dbms_output.put_line('Before dequeue. RUN_ID:USER_ID='||$T_RUN_ID||':'||$T_USER_ID); end;
--/
	INSERT INTO ssstest_user.ssstest_login( run_id, iteration_id, user_id) VALUES( $T_RUN_ID, $T_ITERATION_ID, $T_USER_ID);
	COMMIT;

	declare
  		dequeue_options       dbms_aq.dequeue_options_t;
  		message_properties    dbms_aq.message_properties_t;
  		message_handle        RAW(16);
  		message               ssstest_user.ot_message;
	begin
		-- no need to lock/sleep - dequeue is SYNCRONOUS, i.e. blocking
  		dbms_aq.dequeue(
    			queue_name            => 'ssstest_user.ssstest_queue',
    			dequeue_options       => dequeue_options,
    			message_properties    => message_properties,
    			payload               => message,
    			msgid                 => message_handle);
  		-- debug: dbms_output.enable;
  		-- debug: dbms_output.put_line('Message run_id=' ||message.run_id||', user_id='||message.user_id||', timestamp='||message.lock_datetime);
		:t_run_id 		:= message.run_id;
		:t_iteration_id 	:= message.iteration_id;
		:t_user_id		:= message.user_id;
	end;
/
	-- begin dbms_output.put_line('* Start Run#='|| $T_RUN_ID || ', User#=' || $T_USER_ID || '...'); end;
	begin dbms_output.put_line('* Start Run#='|| :t_run_id || ', User#=' || :t_user_id || '...'); end;
/
	@$QUERY_FILE
	INSERT INTO ssstest_user.ssstest_runstats_messages( run_id, iteration_id, user_id, concurrent_users)
	VALUES( :t_run_id, :t_iteration_id, :t_user_id, $P_CONCURRENT_USERS);
	
	-- Have to use message.user_id instead of $T_USER_ID otherwise messages might be dequeued by a wrong session, so this is incorrect:
	-- VALUES( $T_RUN_ID, $T_ITERATION_ID, $T_USER_ID, $P_CONCURRENT_USERS );
	commit;
	exit
EOF
}

function fnc_exit {
        [[ -f $TMP_PARA_FILE ]] && { rm $TMP_PARA_FILE;  }
        [[ -f $TMP_ALREADY_LOGIN_USERS_FILE ]] && { rm $TMP_ALREADY_LOGIN_USERS_FILE; }
        exit 1
}

#############################################
#############################################
## Main
#############################################
QUERY_FILE=ssstest.qry
PARA_FILE=ssstest.par
EXPONDIST_FILE=/tmp/ssstest_expondist
TMP_PARA_FILE=/tmp/ssstest.$$
TMP_ALREADY_LOGIN_USERS_FILE=/tmp/ssstest_already_login_users.log

[[ -f $QUERY_FILE ]] || { print "Query File is missing. Aborting..."; exit 1; }	
[[ -f $PARA_FILE ]]  || { print "Parameters File is missing. Aborting..."; exit 1; }	
print "Dali Database Solutions Inc. (C) Copyright. All rights reserved. http://www.tns-0.com"
print "Usage: $(basename $0) -- In contrast to the main ssstest.ksh program, this stateLESS vs. stateFULL comparision doesn't accept any arguments";

fnc_process_input_para
fnc_main
fnc_report
fnc_exit

