#!/usr/bin/ksh

# Small utility to troubleshoot response time. Requires a RUN_ID as an input parameter

###############################################
## Function fnc_report
###############################################
function fnc_report {
        $ORACLE_HOME/bin/sqlplus -s /nolog <<EOF
        connect $ORACLE_USERNAME/$ORACLE_PASSWORD
        set lin 150 serveroutput on size 100000 feed off ver off doc off
	col create_datetime		form a30
	col delta			form a30
	col delta_avg			form a30
	col delta_max			form a30
	col delta_min			form a30

	BEGIN dbms_output.put_line( '*** 1) Summary of run# '|| $P_RUN_ID || ' - Max/Min: ***'); END;
/
	  SELECT 
	  	(CASE WHEN run_id < 1000 -- meaning it's stateLESS
             	      THEN (CASE WHEN run_id-100 < 0 THEN run_id ELSE run_id-100 END )
	     	      ELSE (CASE WHEN run_id-1100 < 0 THEN run_id-1000 ELSE run_id-1100 END )
  	  	END
		) run_id,
		concurrent_users,
		(CASE WHEN run_id < 1000
		     THEN 'stateLESS'
		     ELSE 'stateFULL'
		END
		) state,
		-- Avg( Decode( Substr( delta, -6), '000000', 0, delta))  delta_avg, -- no go: Avg doesn't work on intervals - ORA-932
		Max(delta)  delta_max,
		Min( Decode( Substr( delta, -6), '000000', null, delta))  delta_min
	  FROM 	(
		SELECT 	run_id, user_id, concurrent_users, create_datetime, 
		  	(CASE WHEN run_id < 1000 -- meaning it's stateLESS
	             	      THEN (Max(create_datetime) OVER (PARTITION BY (CASE WHEN run_id-100 < 0 THEN run_id ELSE run_id-100 END ), user_id) ) 
		     	      ELSE (Max(create_datetime) OVER (PARTITION BY (CASE WHEN run_id-1100 < 0 THEN run_id-1000 ELSE run_id-1100 END ), user_id) ) 
	  	  	END 
				- create_datetime
		  	) delta
		FROM	ssstest_user.ssstest_runstats_messages
		WHERE	run_id IN ($P_RUN_ID, $P_RUN_ID+100, $P_RUN_ID+1000, $P_RUN_ID+1100)
	  )
	   GROUP BY 
		(CASE WHEN run_id < 1000 -- meaning it's stateLESS
	              THEN (CASE WHEN run_id-100 < 0 THEN run_id ELSE run_id-100 END )
		      ELSE (CASE WHEN run_id-1100 < 0 THEN run_id-1000 ELSE run_id-1100 END )
	  	END
		), 
		concurrent_users,
		(CASE WHEN run_id < 1000
		     THEN 'stateLESS'
		     ELSE 'stateFULL'
		END
		)
	;

/*************************************
	BEGIN dbms_output.put_line( '*** 2) Summary of run# '|| $P_RUN_ID || ' - Min: ***'); END;
/
	  SELECT 
	  	(CASE WHEN run_id < 1000 -- meaning it's stateLESS
             	      THEN (CASE WHEN run_id-100 < 0 THEN run_id ELSE run_id-100 END )
	     	      ELSE (CASE WHEN run_id-1100 < 0 THEN run_id-1000 ELSE run_id-1100 END )
  	  	END
		) run_id,
		concurrent_users,
		(CASE WHEN run_id < 1000
		     THEN 'stateLESS'
		     ELSE 'stateFULL'
		END
		) state,
		Min( Decode( Substr( delta, -6), '000000', null, delta))  delta
	  FROM 	(
		SELECT 	run_id, user_id, concurrent_users, create_datetime, 
		  	(CASE WHEN run_id < 1000 -- meaning it's stateLESS
	             	      THEN (Max(create_datetime) OVER (PARTITION BY (CASE WHEN run_id-100 < 0 THEN run_id ELSE run_id-100 END ), user_id) ) 
		     	      ELSE (Max(create_datetime) OVER (PARTITION BY (CASE WHEN run_id-1100 < 0 THEN run_id-1000 ELSE run_id-1100 END ), user_id) ) 
	  	  	END 
				- create_datetime
		  	) delta
		FROM	ssstest_user.ssstest_runstats_messages
		WHERE	run_id IN ($P_RUN_ID, $P_RUN_ID+100, $P_RUN_ID+1000, $P_RUN_ID+1100)
	  )
	   GROUP BY 
		(CASE WHEN run_id < 1000 -- meaning it's stateLESS
	              THEN (CASE WHEN run_id-100 < 0 THEN run_id ELSE run_id-100 END )
		      ELSE (CASE WHEN run_id-1100 < 0 THEN run_id-1000 ELSE run_id-1100 END )
	  	END
		), 
		concurrent_users,
		(CASE WHEN run_id < 1000
		     THEN 'stateLESS'
		     ELSE 'stateFULL'
		END
		)
	;
**********************************/

	BEGIN dbms_output.put_line( '*** 2) Details of run# '|| $P_RUN_ID ||' (by user): ***'); END;
/
	SELECT	run_id, user_id, concurrent_users, create_datetime, 
		Decode( Substr( delta, -6), '000000', ' ', delta) delta
	FROM 	(
		SELECT 	run_id, user_id, concurrent_users, create_datetime, 
		  	(CASE WHEN run_id < 1000 -- meaning it's stateLESS
	             	      THEN (Max(create_datetime) OVER (PARTITION BY (CASE WHEN run_id-100 < 0 THEN run_id ELSE run_id-100 END ), user_id) ) 
		     	      ELSE (Max(create_datetime) OVER (PARTITION BY (CASE WHEN run_id-1100 < 0 THEN run_id-1000 ELSE run_id-1100 END ), user_id) ) 
	  	  	END 
			- create_datetime
		  	) delta
		FROM	ssstest_user.ssstest_runstats_messages
		WHERE	run_id IN ($P_RUN_ID, $P_RUN_ID+100, $P_RUN_ID+1000, $P_RUN_ID+1100)
	)
	ORDER BY create_datetime;

        exit
EOF
}

###############################################
## Function fnc_process_input_para
###############################################
function fnc_process_input_para {
	grep "^[^#]" $PARA_FILE > $TMP_FILE
	# below typeset is probably not needed (used typeset, as declare is bash specific):
	typeset -i paraLINE_NUMBER=0
	print "Test Parameters Summary ($(date)):"
	# Read 3 uncommented parameter lines (see .par file for detailed structure):
	# sed '/^#/d' $PARA_FILE | 
	while read FULL_LINE ;do
		((paraLINE_NUMBER+=1))
		# print "Line# $paraLINE_NUMBER: $FULL_LINE"
		# LINE_${paraLINE_NUMBER}=$FULL_LINE
		if [ paraLINE_NUMBER -eq 1 ] ;then
			ORACLE_SID=`print $FULL_LINE | cut -f1 -d':'`
			ORACLE_HOME=`print $FULL_LINE | cut -f2 -d':'`
			ORACLE_USERNAME=`print $FULL_LINE | cut -f3 -d':'`
			ORACLE_PASSWORD=`print $FULL_LINE | cut -f4 -d':'`
			# print "$paraLINE_NUMBER: OSID=$ORACLE_SID, OHome=$ORACLE_HOME, OUsername/OPassword=$ORACLE_USERNAME/$ORACLE_PASSWORD"
			export ORACLE_SID ORACLE_HOME ORACLE_USERNAME ORACLE_PASSWORD
		elif [ paraLINE_NUMBER -eq 2 ] ;then
			P_STATE=$(print $FULL_LINE | cut -f1 -d ':')
			P_FREQUENCY_PER_DURATION=$(print $FULL_LINE | cut -f2 -d ':')
			P_DURATION=$(print $FULL_LINE | cut -f3 -d ':')
			P_ITERATIONS=$(print $FULL_LINE | cut -f4 -d ':')
			P_DISTRIBUTION=$(print $FULL_LINE | cut -f5 -d ':')
			P_LET_RUN_AWAY=$(print $FULL_LINE | cut -f6 -d ':')
			P_EXPECTED_SERVICE_TIME=$(print $FULL_LINE | cut -f7 -d ':')
			# print "$paraLINE_NUMBER: STATE=$P_STATE (but doesn't matter for this test), FPD=$P_FREQUENCY_PER_DURATION, DUR=$P_DURATION, ITER=$P_ITERATIONS, DISTR=$P_DISTRIBUTION, RUN_AWAY=$P_LET_RUN_AWAY, EST=$P_EXPECTED_SERVICE_TIME"
		elif [ paraLINE_NUMBER -eq 3 ] ;then

			# do nothing, let next routine figure it out
			set -A runs $(tail -1 $TMP_FILE)
			# Total number of Runs:
			P_RUNS="${#runs[@]}"
			# print "$paraLINE_NUMBER: Number of runs (detailed user ramp-up): $P_RUNS (${runs[@]})"
			[ P_RUNS -gt 5 ] && { print "Number of runs for the stateLESS vs. stateFULL comparision can't exceed 5"; fnc_exit; }
		elif [ paraLINE_NUMBER -eq 4 ] ;then
			print "Problem with the parameter file. Too many uncommented input lines"; exit 1;
		else
			print "Trouble with parsing parameter file - Line# $paraLINE_NUMBER: $FULL_LINE"
		fi
	done < $TMP_FILE
}

function fnc_exit {
        [[ -f $TMP_FILE ]] && { rm $TMP_FILE; }
        exit 1
}

#############################################
#############################################
## Main
#############################################
[ $# -lt 1 ] && { print "Usage: $(basename $0) <RUN_ID>"; exit 1; }

PARA_FILE=ssstest.par
TMP_FILE=/tmp/ssstest.$$

typeset -i P_RUN_ID=$1
# typeset -i P_USER_ID=$2

[[ -f $PARA_FILE ]]  || { print "Parameters File is missing. Aborting..."; exit 1; }	

fnc_process_input_para
## fnc_test_oracle ## work on this
fnc_report
fnc_exit

